@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -12,19 +12,23 @@
 #ifndef GLVIS_AUX_VIS_HPP
 #define GLVIS_AUX_VIS_HPP
 
-#include "gl/platform_gl.hpp"
 #include "gl/types.hpp"
 
 #include "openglvis.hpp"
-#include "sdl.hpp"
+#include "glwindow.hpp"
 #include "font.hpp"
 
 #include <functional>
 
-void SDLMainLoop(bool server_mode = false);
+MainThread& GetMainThread(bool headless = false);
+void MainThreadLoop(bool headless = false, bool server_mode = false);
+
+class GLVisCommand;
+void SetGLVisCommand(GLVisCommand *cmd);
 
 /// Initializes the visualization and some keys.
-int InitVisualization(const char name[], int x, int y, int w, int h);
+GLWindow* InitVisualization(const char name[], int x, int y, int w, int h,
+                            bool headless = false);
 
 void SetVisualizationScene(VisualizationScene * scene,
                            int view = 3, const char *keys = NULL);
@@ -39,31 +43,35 @@ void MyExpose();
 
 void MainLoop();
 
-SdlWindow * GetAppWindow();
+class SdlWindow;
+SdlWindow* GetSdlWindow();
+GLWindow* GetAppWindow();
 VisualizationScene * GetVisualizationScene();
 
 void SetLegacyGLOnly(bool status);
 
 void AddIdleFunc(void (*Func)(void));
 void RemoveIdleFunc(void (*Func)(void));
+void CheckMainIdleFunc();
 
-void LeftButtonDown  (EventInfo *event);
-void LeftButtonLoc   (EventInfo *event);
-void LeftButtonUp    (EventInfo *event);
-void MiddleButtonDown(EventInfo *event);
-void MiddleButtonLoc (EventInfo *event);
-void MiddleButtonUp  (EventInfo *event);
-void RightButtonDown (EventInfo *event);
-void RightButtonLoc  (EventInfo *event);
-void RightButtonUp   (EventInfo *event);
+void LeftButtonDown  (GLWindow::MouseEventInfo *event);
+void LeftButtonLoc   (GLWindow::MouseEventInfo *event);
+void LeftButtonUp    (GLWindow::MouseEventInfo *event);
+void MiddleButtonDown(GLWindow::MouseEventInfo *event);
+void MiddleButtonLoc (GLWindow::MouseEventInfo *event);
+void MiddleButtonUp  (GLWindow::MouseEventInfo *event);
+void RightButtonDown (GLWindow::MouseEventInfo *event);
+void RightButtonLoc  (GLWindow::MouseEventInfo *event);
+void RightButtonUp   (GLWindow::MouseEventInfo *event);
 
 void TouchPinch(SDL_MultiGestureEvent & e);
 
 void KeyCtrlP();
 void KeyS();
+void KeyqPressed();
 void KeyQPressed();
 void ToggleThreads();
-void ThreadsPauseFunc(GLenum);
+void ThreadsPauseFunc(SDL_Keymod);
 void ThreadsStop();
 void ThreadsRun();
 
@@ -77,14 +85,14 @@ void Key7Pressed();
 void Key8Pressed();
 void Key9Pressed();
 
-void Key0Pressed();
-void KeyDeletePressed();
-void KeyEnterPressed();
+void Key0Pressed(SDL_Keymod);
+void KeyDeletePressed(SDL_Keymod);
+void KeyEnterPressed(SDL_Keymod);
 
-void KeyLeftPressed(GLenum);
-void KeyRightPressed(GLenum);
-void KeyUpPressed(GLenum);
-void KeyDownPressed(GLenum);
+void KeyLeftPressed(SDL_Keymod);
+void KeyRightPressed(SDL_Keymod);
+void KeyUpPressed(SDL_Keymod);
+void KeyDownPressed(SDL_Keymod);
 void KeyJPressed();
 void KeyMinusPressed();
 void KeyPlusPressed();
@@ -131,14 +139,16 @@ float GetLineWidthMS();
 
 void InitFont();
 GlVisFont * GetFont();
-bool SetFont(const vector<std::string>& patterns, int height);
+bool SetFont(const std::vector<std::string>& patterns, int height);
 void SetFont(const std::string& fn);
 
 void SetUseHiDPI(bool status);
-function<string(double)> NumberFormatter(int precision=4, char format='d',
-                                         bool showsign=false);
-function<string(double)> NumberFormatter(string formatting);
-bool isValidNumberFormatting(const string& formatting);
+bool GetUseHiDPI();
+
+std::function<std::string(double)> NumberFormatter(int precision=4,
+                                                   char format='d', bool showsign=false);
+std::function<std::string(double)> NumberFormatter(std::string formatting);
+bool isValidNumberFormatting(const std::string& formatting);
 
 // This is a helper function for prompting the user for inputs. The benefit
 // over using just `cin >> input` is that you can specify a type and optionally
@@ -146,22 +156,23 @@ bool isValidNumberFormatting(const string& formatting);
 // True function. If the input cannot be type casted to the expected type, or
 // if it fails the validation, the user is asked again for a new input.
 template <typename T>
-T prompt(const string question,
+T prompt(const std::string question,
          const T* default_value = nullptr,
-function<bool(T)> validator = [](T) { return true; })
+std::function<bool(T)> validator = [](T) { return true; })
 {
    T input;
-   string strInput;
+   std::string strInput;
 
    while (true)
    {
-      cout << question << " ";
-      getline(cin, strInput);
-      stringstream buf(strInput);
+      std::cout << question << " ";
+      std::getline(std::cin, strInput);
+      std::stringstream buf(strInput);
 
       if (strInput.empty() && default_value != nullptr)
       {
-         cout << "Input empty. Using default value: " << *default_value << endl;
+         std::cout << "Input empty. Using default value: " << *default_value
+                   << std::endl;
          return *default_value;
       }
 
@@ -173,15 +184,16 @@ function<bool(T)> validator = [](T) { return true; })
          }
          else
          {
-            cout << "Input is not valid. Please try again." << endl;
+            std::cout << "Input is not valid. Please try again." << std::endl;
          }
       }
       else
       {
-         cout << "Input can not be casted to expected type. Please try again." << endl;
+         std::cout << "Input can not be casted to expected type. Please try again."
+                   << std::endl;
       }
    }
    return input;
 }
 
-#endif
+#endif // GLVIS_AUX_VIS_HPP
