@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -14,6 +14,9 @@
 #include "font.hpp"
 #include "aux_vis.hpp"
 
+using std::cout;
+using std::endl;
+
 struct vert_tex2d
 {
    float x, y;
