@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -21,15 +21,13 @@
 
 #include "gl/platform_gl.hpp"
 
-using namespace std;
-
 class GlVisFont
 {
 public:
    struct glyph
    {
-      uint32_t w, h;
-      int32_t bear_x, bear_y;
+      std::uint32_t w, h;
+      std::int32_t bear_x, bear_y;
       float adv_x, adv_y;
       float tex_x;
    };
@@ -42,7 +40,7 @@ private:
    glyph font_chars[256];
    float tex_w;
    float tex_h;
-   uint32_t font_tex;
+   std::uint32_t font_tex;
 
    FT_Library  library;
    FT_Face     face;
@@ -53,7 +51,7 @@ public:
 
    const glyph &GetTexChar(char c) const
    {
-      return font_chars[(uint8_t) c];
+      return font_chars[(std::uint8_t) c];
    }
 
    /// Get the width and height of the bounding box containing the rendered text
@@ -66,7 +64,7 @@ public:
    {
       if (FT_Init_FreeType(&library))
       {
-         cout << "GLVis: Can not initialize FreeType library!" << endl;
+         std::cout << "GLVis: Can not initialize FreeType library!" << std::endl;
       }
       init = true;
    }
@@ -109,4 +107,4 @@ public:
    void setAlphaChannel(GLenum alpha) { alpha_channel = alpha; }
 };
 
-#endif /* GLVIS_FONT_HPP */
+#endif // GLVIS_FONT_HPP
