@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -11,7 +11,8 @@
 
 #ifndef GLVIS_GEOM_UTILS_HPP
 #define GLVIS_GEOM_UTILS_HPP
-#include "mfem.hpp"
+
+#include <mfem.hpp>
 
 // Some inline functions
 
