@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -12,9 +12,7 @@
 #ifndef GLVIS_ATTR_TRAITS_HPP
 #define GLVIS_ATTR_TRAITS_HPP
 
-#include "types.hpp"
 #include "renderer_core.hpp"
-#include <type_traits>
 
 namespace gl3
 {
@@ -159,4 +157,4 @@ AttrTexcoord<TV, decltype((void)TV::texCoord, 0)>>
 
 }
 
-#endif
+#endif // GLVIS_ATTR_TRAITS_HPP
