@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -14,6 +14,8 @@
 namespace gl3
 {
 
+using namespace resource;
+
 // Beginning in OpenGL 3.0, there were two changes in texture format support:
 // - The older single-channel internal format GL_ALPHA was deprecated in favor
 //   of GL_RED
@@ -148,7 +150,7 @@ void MeshRenderer::render(const RenderQueue& queue)
 
       if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
       {
-         cerr << "Unable to create multisampled renderbuffer." << flush;
+         std::cerr << "Unable to create multisampled renderbuffer." << std::flush;
          glDeleteFramebuffers(1, &fbo);
          glBindFramebuffer(GL_FRAMEBUFFER, 0);
       }
@@ -268,7 +270,7 @@ void MeshRenderer::render(const RenderQueue& queue)
 
       if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
       {
-         cerr << "Unable to create resolve renderbuffer." << endl;
+         std::cerr << "Unable to create resolve renderbuffer." << std::endl;
          glBindFramebuffer(GL_FRAMEBUFFER, 0);
       }
 
