@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -14,8 +14,6 @@
 
 #include <memory>
 #include <vector>
-#include <set>
-#include <unordered_map>
 
 #include "platform_gl.hpp"
 #include "types.hpp"
@@ -25,13 +23,11 @@
 namespace gl3
 {
 
-using namespace resource;
-
 const int LIGHTS_MAX = 3;
 #ifdef GLVIS_MS_LINEWIDTH
 const float LINE_WIDTH_AA = GLVIS_MS_LINEWIDTH;
 #else
-const float LINE_WIDTH_AA = 1.4;
+const float LINE_WIDTH_AA = 1.4f;
 #endif
 
 struct RenderParams
@@ -55,11 +51,11 @@ struct RenderParams
    bool contains_translucent;
 };
 
-typedef vector<pair<RenderParams, GlDrawable*>> RenderQueue;
+typedef std::vector<std::pair<RenderParams, GlDrawable*>> RenderQueue;
 
 struct SceneInfo
 {
-   vector<GlDrawable*> needs_buffering;
+   std::vector<GlDrawable*> needs_buffering;
    RenderQueue queue;
 };
 
@@ -88,9 +84,9 @@ struct FeedbackText
 
 struct CaptureBuffer
 {
-   vector<FeedbackVertex> lines;
-   vector<FeedbackVertex> triangles;
-   vector<FeedbackText> text;
+   std::vector<FeedbackVertex> lines;
+   std::vector<FeedbackVertex> triangles;
+   std::vector<FeedbackText> text;
 };
 
 // OpenGL device interface representing rendering capabilities
@@ -105,7 +101,7 @@ protected:
    std::array<float, 4> static_color;
 
 protected:
-   TextureHandle passthrough_texture;
+   resource::TextureHandle passthrough_texture;
 
 public:
 
@@ -195,7 +191,7 @@ public:
 
 class MeshRenderer
 {
-   unique_ptr<GLDevice> device;
+   std::unique_ptr<GLDevice> device;
    bool msaa_enable;
    int msaa_samples;
    GLuint color_tex, alpha_tex, font_tex;
@@ -243,7 +239,7 @@ public:
          std::cerr << "GL_MAX_SAMPLES = " << msaa_samples
                    << " but requested " << samples << "x MSAA. ";
          std::cerr << "Setting antialiasing mode to "
-                   << msaa_samples << "x MSAA." << endl;
+                   << msaa_samples << "x MSAA." << std::endl;
       }
       else
       {
