@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,13 +9,15 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#include <unordered_map>
+
 #include "attr_traits.hpp"
 #include "renderer_core.hpp"
 #include "../aux_vis.hpp"
 
-#include <regex>
-#include <type_traits>
+#ifdef GLVIS_DEBUG
 #include <unordered_set>
+#endif
 
 // weird but loads them inline
 
@@ -40,6 +42,8 @@ const std::string PRINTING_FS =
 namespace gl3
 {
 
+using namespace std;
+
 const std::vector<std::string> CoreGLDevice::unif_list =
 {
    "useClipPlane",
@@ -146,7 +150,7 @@ void CoreGLDevice::initializeShaderState(const ShaderProgram& prog)
       }
    }
 #ifdef GLVIS_DEBUG
-   unordered_set<string> expectedUnifs(unif_list.begin(), unif_list.end());
+   std::unordered_set<string> expectedUnifs(unif_list.begin(), unif_list.end());
    for (const auto& pairunif : uniforms)
    {
       if (expectedUnifs.find(pairunif.first) == expectedUnifs.end())
@@ -175,12 +179,12 @@ void CoreGLDevice::init()
    {
       GLuint hnd_vao;
       glGenVertexArrays(1, &hnd_vao);
-      global_vao = VtxArrayHandle(hnd_vao);
+      global_vao = resource::VtxArrayHandle(hnd_vao);
       glBindVertexArray(global_vao);
    }
    GLuint hnd_fb_buf;
    glGenBuffers(1, &hnd_fb_buf);
-   feedback_vbo = BufObjHandle(hnd_fb_buf);
+   feedback_vbo = resource::BufObjHandle(hnd_fb_buf);
 }
 
 void CoreGLDevice::setTransformMatrices(glm::mat4 model_view,
