@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -11,11 +11,15 @@
 
 #ifndef GLVIS_RENDERER_CORE_HPP
 #define GLVIS_RENDERER_CORE_HPP
+
+#include <unordered_map>
+
 #include "renderer.hpp"
 #include "shader.hpp"
 
 namespace gl3
 {
+
 // Renderer for OpenGL versions with access to the programmable pipeline
 class CoreGLDevice : public GLDevice
 {
@@ -40,9 +44,9 @@ public:
 private:
    ShaderProgram default_prgm;
    ShaderProgram feedback_prgm;
-   VtxArrayHandle global_vao;
+   resource::VtxArrayHandle global_vao;
 
-   BufObjHandle feedback_vbo;
+   resource::BufObjHandle feedback_vbo;
 
    const static std::vector<std::string> unif_list;
 
@@ -52,8 +56,8 @@ private:
 
    struct VBOData
    {
-      BufObjHandle vert_buf;
-      BufObjHandle elem_buf;
+      resource::BufObjHandle vert_buf;
+      resource::BufObjHandle elem_buf;
       GLenum shape;
       size_t count;
       array_layout layout;
@@ -68,9 +72,9 @@ private:
    void drawDeviceBufferImpl(GLenum shape, int count, bool indexed);
 
    void processTriangleXfbBuffer(CaptureBuffer& cbuf,
-                                 const vector<ShaderXfbVertex>& verts);
+                                 const std::vector<ShaderXfbVertex>& verts);
    void processLineXfbBuffer(CaptureBuffer& cbuf,
-                             const vector<ShaderXfbVertex>& verts);
+                             const std::vector<ShaderXfbVertex>& verts);
 
 public:
    CoreGLDevice()
