@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,6 +9,10 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#include <array>
+#include <vector>
+#include <iostream>
+
 #include "renderer_ff.hpp"
 #include "attr_traits.hpp"
 #include "../aux_vis.hpp"
@@ -192,7 +196,7 @@ void FFGLDevice::bufferToDevice(array_layout layout, IVertexBuffer& buf)
          bufferFFDeviceImpl(static_cast<const VertexBuffer<VertexNormTex>&>(buf));
          break;
       default:
-         cerr << "WARNING: Unhandled vertex layout " << layout << endl;
+         std::cerr << "WARNING: Unhandled vertex layout " << layout << std::endl;
    }
 }
 
@@ -232,7 +236,7 @@ void FFGLDevice::bufferToDevice(array_layout layout, IIndexedBuffer& buf)
          bufferFFDeviceImpl(static_cast<const IndexedVertexBuffer<VertexNormTex>&>(buf));
          break;
       default:
-         cerr << "WARNING: Unhandled vertex layout " << layout << endl;
+         std::cerr << "WARNING: Unhandled vertex layout " << layout << std::endl;
    }
 }
 
@@ -370,7 +374,7 @@ void FFGLDevice::captureXfbBuffer(PaletteState& pal, CaptureBuffer& cbuf,
       return;
    }
    // allocate feedback buffer
-   vector<float> xfb_buf;
+   std::vector<float> xfb_buf;
    xfb_buf.resize(sizebuf);
    glFeedbackBuffer(sizebuf, fbType, xfb_buf.data());
    // draw with feedback capture
