@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,9 +9,12 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
-#include "shader.hpp"
+#include <unordered_map>
 #include <regex>
 
+#include "shader.hpp"
+#include "renderer.hpp"
+
 namespace gl3
 {
 
@@ -269,7 +272,7 @@ bool ShaderProgram::linkShaders(const std::vector<GLuint>& shaders)
    glGetProgramiv(program_id, GL_LINK_STATUS, &stat);
    if (stat == GL_FALSE)
    {
-      cerr << "fatal: Shader linking failed" << endl;
+      std::cerr << "fatal: Shader linking failed" << std::endl;
    }
    return (stat == GL_TRUE);
 }
@@ -283,7 +286,7 @@ void ShaderProgram::mapShaderUniforms()
 
    for (int i = 0; i < num_unifs; i++)
    {
-      vector<char> unif_buf(max_unif_len+1);
+      std::vector<char> unif_buf(max_unif_len+1);
       GLsizei name_length;
       GLint gl_size;
       GLenum gl_type;
