@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -12,10 +12,12 @@
 #ifndef GLVIS_SHADER_HPP
 #define GLVIS_SHADER_HPP
 
-#include "renderer.hpp"
-
+#include <iostream>
 #include <unordered_map>
 
+#include "platform_gl.hpp"
+#include "types.hpp"
+
 namespace gl3
 {
 class ShaderProgram
@@ -69,9 +71,9 @@ private:
    std::unordered_map<int, std::string> attrib_idx;
    int num_outputs = 0;
 
-   ShaderPrgmHandle program_id = 0;
-   ShaderHandle vertex_shader = 0;
-   ShaderHandle fragment_shader = 0;
+   resource::ShaderPrgmHandle program_id = 0;
+   resource::ShaderHandle vertex_shader = 0;
+   resource::ShaderHandle fragment_shader = 0;
    bool is_compiled = false;
    std::unordered_map<std::string, GLuint> uniform_idx;
 };
