@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -14,8 +14,9 @@
 
 #include <vector>
 #include <array>
-#include <iostream>
 #include <memory>
+#include <string>
+#include <cstdint>
 #include <iterator>
 #include <algorithm>
 
@@ -33,8 +34,6 @@
 
 #include "platform_gl.hpp"
 
-using namespace std;
-
 class VisualizationScene;
 
 namespace gl3
