@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -19,4 +19,4 @@ extern unsigned int logo_rgba_len;
 
 #endif // GLVIS_USE_LOGO
 
-#endif //__LOGO_HPP__
+#endif // GLVIS_LOGO_HPP
