@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -12,13 +12,11 @@
 #ifndef GLVIS_OPENGLVIS_HPP
 #define GLVIS_OPENGLVIS_HPP
 
-#include <cmath>
 #include "gl/types.hpp"
 #include "material.hpp"
 #include "palettes.hpp"
-#include "mfem.hpp"
 #include "geom_utils.hpp"
-#include "sdl.hpp"
+#include "glwindow.hpp"
 #include "gltf.hpp"
 
 // Visualization header file
@@ -65,7 +63,7 @@ protected:
    // How to scale the visualized object(s)
    double xscale, yscale, zscale;
 
-   SdlWindow * wnd;
+   GLWindow *wnd;
 
    glm::mat4 proj_mtx;
 
@@ -113,7 +111,7 @@ protected:
       if (val < 0.0) { val = 0.0; }
       if (val > 1.0) { val = 1.0; }
 
-      builder.glTexCoord1f(val);
+      builder.glTexCoord1f(static_cast<float>(val));
    }
 
    // We only need 3 points, but the array is 4x3
@@ -171,7 +169,7 @@ protected:
                 const gl3::GlDrawable &gl_drawable);
 
 public:
-   VisualizationScene();
+   VisualizationScene(GLWindow &wnd);
    virtual ~VisualizationScene();
 
    int spinning, OrthogonalProjection, print, movie;
@@ -211,6 +209,8 @@ public:
    void Scale(double s);
    void Scale(double s1, double s2, double s3);
 
+   virtual void UpdateComplexPhase(double phase) { }
+
    void CenterObject();
    void CenterObject2D();
 
@@ -228,4 +228,4 @@ public:
    int view;
 };
 
-#endif
+#endif // GLVIS_OPENGLVIS_HPP
