@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -29,6 +29,12 @@ public:
    int GetSmoothSetting() { return use_smooth; }
    /// Sets the palette texture to bind.
    void SetIndex(int num);
+   void SetByName(const std::string& palette_name);
+   /// Use the default index if set
+   bool UseDefaultIndex();
+   /// If default index is not set, sets to this index.
+   /// Used for setting recommend palettes
+   void SetFallbackIndex(int idx) { if (!UseDefaultIndex()) { SetIndex(idx); } };
    int GetCurrIndex() const { return curr_palette; }
    void NextIndex();
    void PrevIndex();
