@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -12,20 +12,21 @@
 #ifndef GLVIS_THREADS_HPP
 #define GLVIS_THREADS_HPP
 
-#include "vsdata.hpp"
-#include "data_state.hpp"
 #include <mfem.hpp>
 #include <thread>
 #include <atomic>
 #include <condition_variable>
 
+#include "window.hpp"
+#include "vsdata.hpp"
+#include "data_state.hpp"
+
 class GLVisCommand
 {
 private:
    // Pointers to global GLVis data
-   VisualizationSceneScalarData **vs;
-   DataState&           curr_state;
-   SdlWindow            *thread_wnd;
+   Window      &win;
+   GLWindow    *thread_wnd;
 
    std::mutex glvis_mutex;
    std::condition_variable glvis_cond;
@@ -33,38 +34,41 @@ private:
    int num_waiting;
    bool terminating;
 
-   enum
+   enum class Command
    {
-      NO_COMMAND = 0,
-      NEW_MESH_AND_SOLUTION = 1,
-      SCREENSHOT = 2,
-      KEY_COMMANDS = 3,
-      WINDOW_SIZE = 4,
-      WINDOW_TITLE = 5,
-      PAUSE = 6,
-      VIEW_ANGLES = 7,
-      ZOOM = 8,
-      SUBDIVISIONS = 9,
-      VALUE_RANGE = 10,
-      SHADING = 11,
-      VIEW_CENTER = 12,
-      AUTOSCALE = 13,
-      PALETTE = 14,
-      CAMERA = 15,
-      AUTOPAUSE = 16,
-      WINDOW_GEOMETRY = 17,
-      PLOT_CAPTION = 18,
-      AXIS_LABELS = 19,
-      PALETTE_REPEAT = 20,
-      LEVELLINES = 21,
-      AXIS_NUMBERFORMAT = 22,
-      COLORBAR_NUMBERFORMAT = 23
+      NO_COMMAND,
+      NEW_MESH_AND_SOLUTION,
+      SCREENSHOT,
+      KEY_COMMANDS,
+      WINDOW_SIZE,
+      WINDOW_TITLE,
+      PAUSE,
+      VIEW_ANGLES,
+      ZOOM,
+      SUBDIVISIONS,
+      VALUE_RANGE,
+      SHADING,
+      VIEW_CENTER,
+      AUTOSCALE,
+      PALETTE,
+      PALETTE_NAME,
+      PALETTE_FILE,
+      CAMERA,
+      AUTOPAUSE,
+      WINDOW_GEOMETRY,
+      PLOT_CAPTION,
+      AXIS_LABELS,
+      PALETTE_REPEAT,
+      LEVELLINES,
+      AXIS_NUMBERFORMAT,
+      COLORBAR_NUMBERFORMAT,
+      QUIT
    };
 
    std::atomic<bool> command_ready{false};
 
    // command to be executed
-   int command;
+   Command command;
 
    // command arguments
    DataState     new_state;
@@ -85,6 +89,8 @@ private:
    double        view_center_x, view_center_y;
    std::string   autoscale_mode;
    int           palette, palette_repeat;
+   std::string   palette_name;
+   std::string   palette_file;
    double        lvl_min, lvl_max;
    int           lvl_num;
    double        camera[9];
@@ -101,12 +107,11 @@ private:
 
 public:
    // called by the main execution thread
-   GLVisCommand(VisualizationSceneScalarData **_vs,
-                DataState& thread_state);
+   GLVisCommand(Window &win);
 
    // to be used by worker threads
-   bool KeepAttrib() { return curr_state.keep_attr; } // may need to sync this
-   bool FixElementOrientations() { return curr_state.fix_elem_orient; }
+   bool KeepAttrib() { return win.data_state.keep_attr; } // may need to sync this
+   bool FixElementOrientations() { return win.data_state.fix_elem_orient; }
 
    // called by worker threads
    int NewMeshAndSolution(DataState &&ss);
@@ -126,12 +131,15 @@ public:
    int ViewCenter(double x, double y);
    int Autoscale(const char *mode);
    int Palette(int pal);
+   int PaletteName(std::string palname);
+   int PaletteFile(std::string filename);
    int PaletteRepeat(int n);
    int Levellines(double minv, double maxv, int number);
-   int AxisNumberFormat(string formatting);
-   int ColorbarNumberFormat(string formatting);
+   int AxisNumberFormat(std::string formatting);
+   int ColorbarNumberFormat(std::string formatting);
    int Camera(const double cam[]);
    int Autopause(const char *mode);
+   int Quit();
 
    // called by the main execution thread
    int Execute();
@@ -156,22 +164,28 @@ private:
 
    GLVisCommand* glvis_command;
 
-   // data that may be dynamically allocated by the thread
-   std::unique_ptr<Mesh> new_m;
-   std::unique_ptr<GridFunction> new_g;
-   std::string ident;
-
    // thread object
    std::thread tid;
    // signal for thread cancellation
    std::atomic<bool> terminate_thread {false};
 
+   // flag for closing the window at the end of stream
+   bool end_quit;
+
+   // flag for parallel commands execution thread
+   bool is_multithread;
+
+   static void print_commands();
+   bool execute_one(std::string word);
    void execute();
 
 public:
-   communication_thread(StreamCollection _is, GLVisCommand* cmd);
+   communication_thread(StreamCollection _is, GLVisCommand* cmd,
+                        bool end_quit = false, bool mulithread = true);
+
+   bool process_one();
 
    ~communication_thread();
 };
 
-#endif
+#endif // GLVIS_THREADS_HPP
