@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,22 +9,20 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
-#include <cstdlib>
 #include <cmath>
-
+#include <cstdlib>
 #include <unordered_set>
-
-#include <iomanip>
 #include <sstream>
 #include <limits>
-using namespace std;
 
 #include "vsdata.hpp"
 #include "aux_vis.hpp"
 #include "material.hpp"
 #include "palettes.hpp"
+#include "threads.hpp"
 
-#include "gl2ps.h"
+using namespace std;
+using namespace mfem;
 
 const char *strings_off_on[] = { "off", "on" };
 
@@ -37,7 +35,7 @@ void VisualizationSceneScalarData::FixValueRange()
    {
       // Shading quality may be bad since OpenGL uses single precision. We
       // should probably pre-scale the solution before feeding it to OpenGL
-      int old_prec = cout.precision(12);
+      auto old_prec = std::cout.precision(12);
       cout << "[minv,maxv] = " << "[" << minv << "," << maxv
            << "] (maxv-minv = " << maxv-minv << ")\n --> ";
       minv -= 0.49999e-5*am;
@@ -103,15 +101,15 @@ int VisualizationSceneScalarData::GetAutoRefineFactor()
 
 void VisualizationSceneScalarData::DoAutoscale(bool prepare)
 {
-   if (autoscale == 1)
+   if (autoscale == Autoscale::MeshAndValue)
    {
       FindNewBoxAndValueRange(prepare);
    }
-   else if (autoscale == 2)
+   else if (autoscale == Autoscale::Value)
    {
       FindNewValueRange(prepare);
    }
-   else if (autoscale == 3)
+   else if (autoscale == Autoscale::Mesh)
    {
       FindMeshBox(prepare);
    }
@@ -119,7 +117,7 @@ void VisualizationSceneScalarData::DoAutoscale(bool prepare)
 
 void VisualizationSceneScalarData::DoAutoscaleValue(bool prepare)
 {
-   if (autoscale == 1 || autoscale == 3)
+   if (autoscale == Autoscale::MeshAndValue || autoscale == Autoscale::Mesh)
    {
       FindNewBoxAndValueRange(prepare);
    }
@@ -596,13 +594,13 @@ void VisualizationSceneScalarData::PrepareColorBar (double minval,
 // Draw a centered caption at the top (visible with the colorbar)
 void VisualizationSceneScalarData::PrepareCaption()
 {
-   bool empty = plot_caption.empty();
+   bool empty = win.plot_caption.empty();
    colorbar = (colorbar ? empty+1 : !empty);
 
-   string caption(plot_caption);
-   if (!extra_caption.empty())
+   string caption(win.plot_caption);
+   if (!win.extra_caption.empty())
    {
-      caption += " (" + extra_caption + ")";
+      caption += " (" + win.extra_caption + ")";
    }
 
    caption_buf.clear();
@@ -611,8 +609,8 @@ void VisualizationSceneScalarData::PrepareCaption()
    GetFont()->getObjectSize(caption, caption_w, caption_h);
 }
 
-thread_local VisualizationSceneScalarData * vsdata;
-extern thread_local VisualizationScene  * locscene;
+static thread_local VisualizationSceneScalarData *vsdata;
+static thread_local Window *window;
 
 void KeycPressed(GLenum state)
 {
@@ -643,7 +641,7 @@ void KeycPressed(GLenum state)
 void KeyCPressed()
 {
    cout << "Enter new caption: " << flush;
-   std::getline(cin, plot_caption);
+   std::getline(cin, window->plot_caption);
    vsdata->PrepareCaption(); // turn on or off the caption
    SendExposeEvent();
 }
@@ -664,11 +662,12 @@ void Key_Mod_a_Pressed(GLenum state)
 {
    if (state & KMOD_CTRL)
    {
-      static const char *autoscale_modes[] = { "off", "on", "value", "mesh" };
-      int autoscale = vsdata->GetAutoscale();
-      autoscale = (autoscale + 1)%4;
+      constexpr int nmodes = (int) VisualizationSceneScalarData::Autoscale::Max;
+      static const char *autoscale_modes[nmodes] = { "off", "on", "value", "mesh" };
+      int autoscale = (int)vsdata->GetAutoscale();
+      autoscale = (autoscale + 1) % (nmodes);
       cout << "Autoscale: " << flush;
-      vsdata->SetAutoscale(autoscale);
+      vsdata->SetAutoscale((VisualizationSceneScalarData::Autoscale) autoscale);
       cout << autoscale_modes[autoscale] << endl;
       SendExposeEvent();
    }
@@ -715,22 +714,22 @@ void KeyLPressed()
 
 void KeyrPressed()
 {
-   locscene -> spinning = 0;
+   window->vs -> spinning = 0;
    RemoveIdleFunc(MainLoop);
    vsdata -> CenterObject();
 
-   locscene -> ViewAngle = 45.0;
-   locscene -> ViewScale = 1.0;
-   locscene -> ViewCenterX = 0.0;
-   locscene -> ViewCenterY = 0.0;
-   locscene->cam.Reset();
+   window->vs -> ViewAngle = 45.0;
+   window->vs -> ViewScale = 1.0;
+   window->vs -> ViewCenterX = 0.0;
+   window->vs -> ViewCenterY = 0.0;
+   window->vs->cam.Reset();
    vsdata -> key_r_state = 0;
    SendExposeEvent();
 }
 
 void KeyRPressed()
 {
-   locscene->spinning = 0;
+   window->vs->spinning = 0;
    RemoveIdleFunc(MainLoop);
    vsdata->Toggle2DView();
    SendExposeEvent();
@@ -744,14 +743,14 @@ void KeypPressed(GLenum state)
    }
    else
    {
-      locscene->palette.NextIndex();
+      window->vs->palette.NextIndex();
       SendExposeEvent();
    }
 }
 
 void KeyPPressed()
 {
-   locscene->palette.PrevIndex();
+   window->vs->palette.PrevIndex();
    SendExposeEvent();
 }
 
@@ -910,30 +909,30 @@ void KeyF2Pressed()
 
 void KeykPressed()
 {
-   locscene->matAlpha -= 0.05;
-   if (locscene->matAlpha < 0.0)
+   window->vs->matAlpha -= 0.05;
+   if (window->vs->matAlpha < 0.0)
    {
-      locscene->matAlpha = 0.0;
+      window->vs->matAlpha = 0.0;
    }
-   locscene->GenerateAlphaTexture();
+   window->vs->GenerateAlphaTexture();
    SendExposeEvent();
 }
 
 void KeyKPressed()
 {
-   locscene->matAlpha += 0.05;
-   if (locscene->matAlpha > 1.0)
+   window->vs->matAlpha += 0.05;
+   if (window->vs->matAlpha > 1.0)
    {
-      locscene->matAlpha = 1.0;
+      window->vs->matAlpha = 1.0;
    }
-   locscene->GenerateAlphaTexture();
+   window->vs->GenerateAlphaTexture();
    SendExposeEvent();
 }
 
 void KeyAPressed()
 {
-   bool curr_aa = GetAppWindow()->getRenderer().getAntialiasing();
-   GetAppWindow()->getRenderer().setAntialiasing(!curr_aa);
+   bool curr_aa = window->wnd->getRenderer().getAntialiasing();
+   window->wnd->getRenderer().setAntialiasing(!curr_aa);
 
    cout << "Multisampling/Antialiasing: "
         << strings_off_on[!curr_aa ? 1 : 0] << endl;
@@ -944,23 +943,23 @@ void KeyAPressed()
 
 void KeyCommaPressed()
 {
-   locscene->matAlphaCenter -= 0.25;
+   window->vs->matAlphaCenter -= 0.25;
    // vsdata -> EventUpdateColors();
-   locscene->GenerateAlphaTexture();
+   window->vs->GenerateAlphaTexture();
    SendExposeEvent();
 #ifdef GLVIS_DEBUG
-   cout << "MatAlphaCenter = " << locscene->matAlphaCenter << endl;
+   cout << "MatAlphaCenter = " << window->vs->matAlphaCenter << endl;
 #endif
 }
 
 void KeyLessPressed()
 {
-   locscene->matAlphaCenter += 0.25;
+   window->vs->matAlphaCenter += 0.25;
    // vsdata -> EventUpdateColors();
-   locscene->GenerateAlphaTexture();
+   window->vs->GenerateAlphaTexture();
    SendExposeEvent();
 #ifdef GLVIS_DEBUG
-   cout << "MatAlphaCenter = " << locscene->matAlphaCenter << endl;
+   cout << "MatAlphaCenter = " << window->vs->matAlphaCenter << endl;
 #endif
 }
 
@@ -1160,7 +1159,7 @@ void VisualizationSceneScalarData::Toggle2DView()
          break;
    }
 
-   // if (locscene -> view != 2) // make 'R' work the same in 2D and 3D
+   // if (window->vs -> view != 2) // make 'R' work the same in 2D and 3D
    key_r_state = (key_r_state+1)%6;
 
    rotmat = newrot.mtx;
@@ -1344,30 +1343,37 @@ void VisualizationSceneScalarData::ToggleTexture()
    }
 }
 
-void VisualizationSceneScalarData::SetAutoscale(int _autoscale)
+void VisualizationSceneScalarData::SetAutoscale(Autoscale _autoscale,
+                                                bool update)
 {
    if (autoscale != _autoscale)
    {
       autoscale = _autoscale;
-      DoAutoscale(true);
+      if (update)
+      {
+         DoAutoscale(true);
+      }
    }
 }
 
 VisualizationSceneScalarData::VisualizationSceneScalarData(
-   Mesh & m, Vector & s, Mesh *mc)
-   : a_label_x("x"), a_label_y("y"), a_label_z("z")
+   Window &win_, bool init) : VisualizationScene(*win_.wnd), win(win_)
 {
-   mesh = &m;
-   mesh_coarse = mc;
-   sol  = &s;
+   mesh = win.data_state.mesh.get();
+   mesh_coarse = win.data_state.mesh_quad.get();
+   sol  = win.data_state.sol.get();
+   offsets = win.data_state.offsets.get();
 
-   Init();
+   if (init)
+   {
+      Init();
+   }
 }
 
 void VisualizationSceneScalarData::Init()
 {
    vsdata = this;
-   wnd = GetAppWindow();
+   window = &win;
 
    arrow_type = arrow_scaling_type = 0;
    scaling = 0;
@@ -1466,7 +1472,7 @@ void VisualizationSceneScalarData::Init()
 
    PrepareRuler();
 
-   autoscale = 1;
+   autoscale = VisualizationSceneScalarData::Autoscale::MeshAndValue;
 }
 
 VisualizationSceneScalarData::~VisualizationSceneScalarData()
@@ -1749,7 +1755,7 @@ void VisualizationSceneScalarData::SetLevelLines (
 
 void VisualizationSceneScalarData::PrintState()
 {
-   cout << "\nkeys: " << GetAppWindow()->getSavedKeys() << "\n"
+   cout << "\nkeys: " << wnd->getSavedKeys() << "\n"
         << "\nlight " << strings_off_on[use_light ? 1 : 0]
         << "\nperspective " << strings_off_on[OrthogonalProjection ? 0 : 1]
         << "\nviewcenter " << ViewCenterX << ' ' << ViewCenterY
