@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -12,16 +12,10 @@
 #ifndef GLVIS_VSDATA_HPP
 #define GLVIS_VSDATA_HPP
 
-#include <array>
-
-#include "mfem.hpp"
+#include <mfem.hpp>
 #include "openglvis.hpp"
 #include "aux_vis.hpp"
-
-using namespace mfem;
-
-extern thread_local std::string plot_caption; // defined in glvis.cpp
-extern thread_local std::string extra_caption; // defined in glvis.cpp
+#include "window.hpp"
 
 class Plane
 {
@@ -68,23 +62,43 @@ public:
       Max
    };
 
+   // autoscale controls the behavior when the mesh/solution are updated
+   enum class Autoscale
+   {
+      Invalid = -1,
+      Min = -1,
+      //---------
+      None, // do not change the bounding box and the value range
+      MeshAndValue, // recompute both the bounding box and the value range (default)
+      Value, // recompute only the value range
+      Mesh, // recompute only the bounding box
+      //---------
+      Max
+   };
+
 protected:
-   Mesh   *mesh{}, *mesh_coarse{};
-   Vector *sol{};
+   mfem::Mesh   *mesh{}, *mesh_coarse{};
+   mfem::Vector *sol{};
+   const DataState::Offsets *offsets{};
+
+   Window &win;
 
    double minv, maxv;
 
-   std::string a_label_x, a_label_y, a_label_z;
+   std::string a_label_x{"x"}, a_label_y{"y"}, a_label_z{"z"};
 
    int scaling, colorbar, drawaxes;
    Shading shading;
    int auto_ref_max, auto_ref_min_surf_vert, auto_ref_max_surf_vert;
+   bool legacy_parallel_numbering = false;
 
    // Formatter for axes & colorbar numbers. Set defaults.
-   function<string(double)> axis_formatter = NumberFormatter(4, 'd', false);
-   function<string(double)> colorbar_formatter = NumberFormatter(4, 'd', false);
+   std::function<std::string(double)> axis_formatter
+      = NumberFormatter(4, 'd', false);
+   std::function<std::string(double)> colorbar_formatter
+      = NumberFormatter(4, 'd', false);
 
-   vector<gl3::GlDrawable*> updated_bufs;
+   std::vector<gl3::GlDrawable*> updated_bufs;
    gl3::GlDrawable axes_buf;
    gl3::GlDrawable coord_cross_buf;
    gl3::GlDrawable color_bar;
@@ -97,17 +111,12 @@ protected:
    int arrow_type, arrow_scaling_type;
 
    int nl;
-   Array<double> level;
+   mfem::Array<double> level;
 
    int ruler_on;
    double ruler_x, ruler_y, ruler_z;
 
-   // autoscale controls the behavior when the mesh/solution are updated:
-   // 0 - do not change the bounding box and the value range
-   // 1 - recompute both the bounding box and the value range (default)
-   // 2 - recompute only the value range
-   // 3 - recompute only the bounding box
-   int autoscale;
+   Autoscale autoscale;
 
    bool logscale;
 
@@ -143,7 +152,7 @@ protected:
 
    void FixValueRange();
 
-   static int GetFunctionAutoRefineFactor(GridFunction &gf);
+   static int GetFunctionAutoRefineFactor(mfem::GridFunction &gf);
    virtual int GetFunctionAutoRefineFactor() = 0;
    virtual int GetAutoRefineFactor();
 
@@ -158,12 +167,13 @@ public:
    /// Shrink factor with respect to the element (material) attributes centers
    double shrinkmat;
 
-   VisualizationSceneScalarData()
-      : a_label_x("x"), a_label_y("y"), a_label_z("z") {}
-   VisualizationSceneScalarData (Mesh & m, Vector & s, Mesh *mc = NULL);
+   VisualizationSceneScalarData(Window &win, bool init = true);
 
    virtual ~VisualizationSceneScalarData();
 
+   /// Set a new mesh and solution from the given data state
+   virtual void NewMeshAndSolution(const DataState &s) = 0;
+
    virtual std::string GetHelpString() const { return ""; }
 
    // Determine 'xscale', 'yscale', and 'zscale' using the current bounding
@@ -187,10 +197,10 @@ public:
    virtual void FindMeshBox(bool prepare) { FindNewBox(prepare); }
 
    // Perform autoscaling depending on the value of 'autoscale':
-   // 0 - do nothing
-   // 1 - call FindNewBoxAndValueRange
-   // 2 - call FindNewValueRange
-   // 3 - call FindMeshBox
+   // None - do nothing
+   // MeshAndValue - call FindNewBoxAndValueRange
+   // Value - call FindNewValueRange
+   // Mesh - call FindMeshBox
    void DoAutoscale(bool prepare);
    // Similar to the above but force recomputation of the value range
    void DoAutoscaleValue(bool prepare);
@@ -215,13 +225,15 @@ public:
       auto_ref_max_surf_vert = max_surf_vert;
    }
    virtual void AutoRefine() = 0;
-   virtual void ToggleAttributes(Array<int> &attr_list) = 0;
+   virtual void ToggleAttributes(mfem::Array<int> &attr_list) = 0;
 
    virtual void PrintState();
 
-   Mesh *GetMesh() { return mesh; }
+   mfem::Mesh *GetMesh() { return mesh; }
 
-   virtual gl3::SceneInfo GetSceneObjs();
+   gl3::SceneInfo GetSceneObjs() override;
+
+   void UpdateComplexPhase(double ph) override { win.UpdateComplexPhase(ph); }
 
    void ProcessUpdatedBufs(gl3::SceneInfo& scene);
 
@@ -262,7 +274,7 @@ public:
                double cval = HUGE_VAL);
 
    void DrawPolygonLevelLines(gl3::GlBuilder& builder, double *point, int n,
-                              Array<double> &level, bool log_vals);
+                              mfem::Array<double> &level, bool log_vals);
 
    void ToggleLight() { use_light = !use_light; }
    void SetLight(bool light_set) { use_light = light_set; }
@@ -272,23 +284,23 @@ public:
       // colorbar states are: 0) no colorbar, no caption; 1) colorbar with
       // caption; 2) colorbar without caption.
       static const int next[2][3] = { { 1, 2, 0 }, { 2, 0, 0 } };
-      colorbar = next[plot_caption.empty()][colorbar];
+      colorbar = next[win.plot_caption.empty()][colorbar];
    }
 
    // Turn on or off the caption
    void PrepareCaption();
 
    void SetColorbarNumberFormat(int precision, char format, bool showsign);
-   void SetColorbarNumberFormat(string formatting);
+   void SetColorbarNumberFormat(std::string formatting);
 
    void PrepareColorBar(double minval, double maxval,
-                        Array<double> * level = NULL,
-                        Array<double> * levels = NULL);
+                        mfem::Array<double> * level = nullptr,
+                        mfem::Array<double> * levels = nullptr);
 
    void SetAxisLabels(const char * a_x, const char * a_y, const char * a_z);
 
    void SetAxisNumberFormat(int precision, char format, bool showsign);
-   void SetAxisNumberFormat(string formatting);
+   void SetAxisNumberFormat(std::string formatting);
 
    void PrepareAxes();
    void ToggleDrawAxes()
@@ -314,13 +326,13 @@ public:
 
    void Toggle2DView();
 
-   void SetAutoscale(int _autoscale);
-   int GetAutoscale() const { return autoscale; }
+   void SetAutoscale(Autoscale _autoscale, bool update = true);
+   Autoscale GetAutoscale() const { return autoscale; }
 
    /// Shrink the set of points towards attributes centers of gravity
-   void ShrinkPoints(DenseMatrix &pointmat, int i, int fn, int di);
+   void ShrinkPoints(mfem::DenseMatrix &pointmat, int i, int fn, int di);
    // Centers of gravity based on the boundary/element attributes
-   DenseMatrix bdrc, matc;
+   mfem::DenseMatrix bdrc, matc;
    /// Compute the center of gravity for each boundary attribute
    void ComputeBdrAttrCenter();
    /// Compute the center of gravity for each element attribute
