@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,24 +9,16 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
-#include <cstdlib>
-#include <iostream>
-#include <string>
-#include <limits>
-#include <cmath>
-#include <vector>
+#include "vssolution.hpp"
 
-#include "mfem.hpp"
-#include "visual.hpp"
+#include "threads.hpp"
 #include "palettes.hpp"
 #include "gltf.hpp"
 
-using namespace mfem;
 using namespace std;
-
+using namespace mfem;
 
 thread_local VisualizationSceneSolution *vssol;
-extern thread_local VisualizationScene  *locscene;
 extern thread_local GeometryRefiner GLVisGeometryRefiner;
 
 #ifdef GLVIS_ISFINITE
@@ -75,9 +67,16 @@ std::string VisualizationSceneSolution::GetHelpString() const
       << "| o -  (De)refine elem. (NC shading) |" << endl
       << "| O -  Switch 'o' func. (NC shading) |" << endl
       << "| p/P  Cycle through color palettes  |" << endl
-      << "| q -  Quits                         |" << endl
-      << "| Q -  Cycle quadrature data mode    |" << endl
-      << "| r -  Reset the plot to 3D view     |" << endl
+      << "| q -  Quits                         |" << endl;
+   if (win.data_state.quad_f)
+   {
+      os << "| Q -  Cycle quadrature data mode    |" << endl;
+   }
+   else if (win.data_state.cgrid_f)
+   {
+      os << "| Q -  Cycle complex data mode       |" << endl;
+   }
+   os << "| r -  Reset the plot to 3D view     |" << endl
       << "| R -  Reset the plot to 2D view     |" << endl
       << "| s -  Turn on/off unit cube scaling |" << endl
       << "| S -  Take snapshot/Record a movie  |" << endl
@@ -87,6 +86,7 @@ std::string VisualizationSceneSolution::GetHelpString() const
       << "| \\ -  Set light source position     |" << endl
       << "| Alt+a  - Axes number format        |" << endl
       << "| Alt+c  - Colorbar number format    |" << endl
+      << "| Alt+n  - Numberings method         |" << endl
       << "| Ctrl+o - Element ordering curve    |" << endl
       << "| Ctrl+p - Print to a PDF file       |" << endl
       << "+------------------------------------+" << endl
@@ -108,8 +108,13 @@ std::string VisualizationSceneSolution::GetHelpString() const
       << "| *,/  Scale up/down                 |" << endl
       << "| +/-  Change z-scaling              |" << endl
       << "| . -  Start/stop spinning           |" << endl
-      << "| 0/Enter - Spinning speed and dir.  |" << endl
-      << "+------------------------------------+" << endl
+      << "| 0/Enter - Spinning speed and dir.  |" << endl;
+   if (win.data_state.cgrid_f)
+   {
+      os << "| Alt+. -  Start/stop phase anim.    |" << endl
+         << "| Alt+0/Enter - Phase anim. speed    |" << endl;
+   }
+   os << "+------------------------------------+" << endl
       << "| Mouse                              |" << endl
       << "+------------------------------------+" << endl
       << "| left   btn    - Rotation           |" << endl
@@ -191,6 +196,7 @@ static void SwitchAttribute(int increment, int &attribute,
    }
    else
    {
+      vssol->PrepareNumbering();
       vssol->PrepareLines();
       vssol->Prepare();
    }
@@ -279,9 +285,20 @@ static void KeyMPressed()
    SendExposeEvent();
 }
 
-static void KeyNPressed()
+static void KeyNPressed(GLenum state)
 {
-   vssol -> ToggleDrawNumberings();
+   if (state & KMOD_ALT)
+   {
+      vssol->ToggleParallelNumbering();
+   }
+   else if (state & (KMOD_CTRL | KMOD_GUI))
+   {
+      /* No-op */
+   }
+   else
+   {
+      vssol->ToggleDrawNumberings();
+   }
    SendExposeEvent();
 }
 
@@ -419,32 +436,36 @@ static void KeyF12Pressed()
    }
 }
 
-VisualizationSceneSolution::VisualizationSceneSolution()
-{
-   v_normals = NULL;
-}
-
 VisualizationSceneSolution::VisualizationSceneSolution(
-   Mesh &m, Vector &s, Mesh *mc, Vector *normals)
+   Window &win_, bool init) : VisualizationSceneScalarData(win_, false)
 {
-   mesh = &m;
-   mesh_coarse = mc;
-   sol = &s;
-   v_normals = normals;
+   v_normals = win.data_state.normals.get();
 
-   Init();
+   if (win.data_state.grid_f)
+   {
+      rsol = win.data_state.grid_f.get();
+      sol = new Vector(mesh->GetNV());
+   }
+
+   if (init)
+   {
+      if (rsol)
+      {
+         rsol->GetNodalValues(*sol);
+      }
+      Init();
+   }
 }
 
 void VisualizationSceneSolution::Init()
 {
-   rsol  = NULL;
    vssol = this;
 
    drawelems = 1;
    shading = Shading::Smooth;
    drawmesh  = 0;
    draworder = 0;
-   drawnums  = 0;
+   drawnums  = Numbering::NONE;
 
    refine_func = 0;
    have_sol_range = false;
@@ -467,7 +488,7 @@ void VisualizationSceneSolution::Init()
 
    VisualizationSceneScalarData::Init();  // Calls FindNewBox() !!!
 
-   palette.SetIndex(2); // use the 'jet-like' palette in 2D
+   palette.SetFallbackIndex(2); // use the 'jet-like' palette in 2D
 
    double eps = 1e-6; // move the cutting plane a bit to avoid artifacts
    CuttingPlane = new Plane(-1.0,0.0,0.0,(0.5-eps)*bb.x[0]+(0.5+eps)*bb.x[1]);
@@ -523,6 +544,10 @@ void VisualizationSceneSolution::Init()
 
 VisualizationSceneSolution::~VisualizationSceneSolution()
 {
+   if (rsol)
+   {
+      delete sol;
+   }
 }
 
 void VisualizationSceneSolution::ToggleDrawElems()
@@ -549,11 +574,11 @@ void VisualizationSceneSolution::ToggleDrawElems()
 
    if (drawelems < 2)
    {
-      extra_caption.clear();
+      win.extra_caption.clear();
    }
    else
    {
-      extra_caption = modes[drawelems];
+      win.extra_caption = modes[drawelems];
    }
 
    if (drawelems == 0)
@@ -586,6 +611,19 @@ void VisualizationSceneSolution::ToggleDrawElems()
    }
 }
 
+void VisualizationSceneSolution::NewMeshAndSolution(const DataState &s)
+{
+   if (rsol && s.grid_f)
+   {
+      s.grid_f->GetNodalValues(*sol);
+      NewMeshAndSolution(s.mesh.get(), s.mesh_quad.get(), sol, s.grid_f.get());
+   }
+   else
+   {
+      NewMeshAndSolution(s.mesh.get(), s.mesh_quad.get(), s.sol.get());
+   }
+}
+
 void VisualizationSceneSolution::NewMeshAndSolution(
    Mesh *new_m, Mesh *new_mc, Vector *new_sol, GridFunction *new_u)
 {
@@ -594,6 +632,8 @@ void VisualizationSceneSolution::NewMeshAndSolution(
    mesh_coarse = new_mc;
    sol = new_sol;
    rsol = new_u;
+   MFEM_VERIFY(new_sol->Size() == mesh->GetNV(),
+               "New solution vector size does not match the mesh node count.");
 
    // If the number of elements changes, recompute the refinement factor
    if (mesh->GetNE() != old_m->GetNE())
@@ -619,7 +659,6 @@ void VisualizationSceneSolution::NewMeshAndSolution(
    PrepareOrderingCurve();
 }
 
-
 void VisualizationSceneSolution::GetRefinedDetJ(
    int i, const IntegrationRule &ir, Vector &vals, DenseMatrix &tr)
 {
@@ -697,8 +736,10 @@ void VisualizationSceneSolution::GetRefinedDetJ(
    J.ClearExternalData();
 }
 
-void VisualizationSceneSolution::GetRefinedValues(
-   int i, const IntegrationRule &ir, Vector &vals, DenseMatrix &tr)
+void VisualizationSceneSolution::GetRefinedValues(const int i,
+                                                  const IntegrationRule &ir,
+                                                  Vector &vals, DenseMatrix &tr,
+                                                  const bool do_shrink)
 {
    if (drawelems < 2)
    {
@@ -715,15 +756,16 @@ void VisualizationSceneSolution::GetRefinedValues(
          vals(j) = _LogVal(vals(j));
       }
 
-   if (shrink != 1.0 || shrinkmat != 1.0)
+   if (do_shrink && (shrink != 1.0 || shrinkmat != 1.0))
    {
       ShrinkPoints(tr, i, 0, 0);
    }
 }
 
-int VisualizationSceneSolution::GetRefinedValuesAndNormals(
-   int i, const IntegrationRule &ir, Vector &vals, DenseMatrix &tr,
-   DenseMatrix &normals)
+int VisualizationSceneSolution::GetRefinedValuesAndNormals(const int i,
+                                                           const IntegrationRule &ir,
+                                                           Vector &vals, DenseMatrix &tr,
+                                                           DenseMatrix &normals)
 {
    int have_normals = 0;
 
@@ -1122,8 +1164,14 @@ void VisualizationSceneSolution::FindNewBox(bool prepare)
 {
    FindNewBox(bb.x, bb.y, bb.z);
 
-   minv = bb.z[0];
-   maxv = bb.z[1];
+   win.data_state.FindValueRange(minv, maxv,
+   [this](double v) { return LogVal(v); });
+
+   if (minv == 0. && maxv == 0.)
+   {
+      minv = bb.z[0];
+      maxv = bb.z[1];
+   }
 
    FixValueRange();
 
@@ -1136,11 +1184,16 @@ void VisualizationSceneSolution::FindNewBox(bool prepare)
 
 void VisualizationSceneSolution::FindNewValueRange(bool prepare)
 {
-   double rx[2], ry[2], rv[2];
+   win.data_state.FindValueRange(minv, maxv,
+   [this](double v) { return LogVal(v); });
 
-   FindNewBox(rx, ry, rv);
-   minv = rv[0];
-   maxv = rv[1];
+   if (minv == 0. && maxv == 0.)
+   {
+      double rx[2], ry[2], rv[2];
+      FindNewBox(rx, ry, rv);
+      minv = rv[0];
+      maxv = rv[1];
+   }
 
    FixValueRange();
 
@@ -1825,169 +1878,156 @@ double VisualizationSceneSolution::GetElementLengthScale(int k)
 
 void VisualizationSceneSolution::PrepareElementNumbering()
 {
+   auto offset = [&](const int e)
+   {
+      if (!offsets) { return e; }
+      if (legacy_parallel_numbering) { return e; }
+      const int rank = mesh->GetAttribute(e) - 1;
+      MFEM_VERIFY(rank >= 0 && rank < (int)offsets->size(),
+                  "Invalid rank for element " + std::to_string(e));
+      const int nelems = (*offsets)[rank].nelems;
+      MFEM_VERIFY(e >= nelems,
+                  "Invalid element " + std::to_string(e) +
+                  " for rank " + std::to_string(rank));
+      return e - nelems;
+   };
+
+   e_nums_buf.clear();
+
    if (shading == Shading::Noncomforming)
    {
-      PrepareElementNumbering2();
+      IntegrationRule center_ir(1);
+      DenseMatrix pointmat;
+      Vector values;
+
+      for (int e = 0; e < mesh->GetNE(); e++)
+      {
+         if (!el_attr_to_show[mesh->GetAttribute(e)-1]) { continue; }
+         center_ir.IntPoint(0) =
+            Geometries.GetCenter(mesh->GetElementBaseGeometry(e));
+         GetRefinedValues(e, center_ir, values, pointmat);
+         const double xc = pointmat(0,0), yc = pointmat(1,0), uc = values(0);
+         const double dx = 0.05 * GetElementLengthScale(e);
+         const double xx[3] = {xc, yc, uc};
+         DrawNumberedMarker(e_nums_buf, xx, dx, offset(e));
+#ifdef GLVIS_DEBUG
+         if (offsets && shrink == 1.0 && shrinkmat == 1.0)
+         {
+            constexpr double eps = 1e-12;
+            const int rank = mesh->GetAttribute(e) - 1;
+            MFEM_VERIFY(rank >= 0 && rank < (int)offsets->size(),
+                        "Invalid rank");
+            const auto &xy = (*offsets)[rank].exy_map.at({e,rank});
+            MFEM_VERIFY(fabs(xy.x - xc) < eps && fabs(xy.y - yc) < eps,
+                        "Element center does not match expected position");
+         }
+#endif // GLVIS_DEBUG
+      }
    }
    else
    {
-      PrepareElementNumbering1();
-   }
-}
+      DenseMatrix pointmat;
+      Array<int> vertices;
 
-void VisualizationSceneSolution::PrepareElementNumbering1()
-{
-   e_nums_buf.clear();
-
-   DenseMatrix pointmat;
-   Array<int> vertices;
-
-   int ne = mesh->GetNE();
-   for (int k = 0; k < ne; k++)
-   {
-      mesh->GetPointMatrix (k, pointmat);
-      mesh->GetElementVertices (k, vertices);
-      int nv = vertices.Size();
-
-      ShrinkPoints(pointmat, k, 0, 0);
-
-      double xs = 0.0;
-      double ys = 0.0;
-      double us = 0.0;
-      for (int j = 0; j < nv; j++)
+      for (int e = 0; e < mesh->GetNE(); e++)
       {
-         xs += pointmat(0,j);
-         ys += pointmat(1,j);
-         us += LogVal((*sol)(vertices[j]));
+         if (!el_attr_to_show[mesh->GetAttribute(e) - 1]) { continue; }
+         mesh->GetPointMatrix(e, pointmat);
+         mesh->GetElementVertices(e, vertices);
+         const int nv = vertices.Size();
+         ShrinkPoints(pointmat, e, 0, 0);
+         double xs = 0.0, ys = 0.0, us = 0.0;
+         for (int j = 0; j < nv; j++)
+         {
+            xs += pointmat(0,j);
+            ys += pointmat(1,j);
+            us += LogVal((*sol)(vertices[j]));
+         }
+         xs /= nv;
+         ys /= nv;
+         us /= nv;
+#ifdef GLVIS_DEBUG
+         if (offsets && shrink == 1.0 && shrinkmat == 1.0)
+         {
+            constexpr double eps = 1e-12;
+            const int rank = mesh->GetAttribute(e) - 1;
+            MFEM_VERIFY(rank >= 0 && rank < (int)offsets->size(),
+                        "Invalid rank");
+            const auto &xy = (*offsets)[rank].exy_map.at({e, rank});
+            MFEM_VERIFY(fabs(xy.x - xs) < eps && fabs(xy.y - ys) < eps,
+                        "Element center does not match expected position");
+         }
+#endif // GLVIS_DEBUG
+         const double dx = 0.05 * GetElementLengthScale(e);
+         const double xx[3] = {xs, ys, us};
+         DrawNumberedMarker(e_nums_buf, xx, dx, offset(e));
       }
-      xs /= nv;
-      ys /= nv;
-      us /= nv;
-
-      double ds = GetElementLengthScale(k);
-      double dx = 0.05*ds;
-
-      double xx[3] = {xs,ys,us};
-      DrawNumberedMarker(e_nums_buf,xx,dx,k);
    }
-
-   updated_bufs.emplace_back(&e_nums_buf);
-}
-
-void VisualizationSceneSolution::PrepareElementNumbering2()
-{
-   IntegrationRule center_ir(1);
-   DenseMatrix pointmat;
-   Vector values;
-
-   e_nums_buf.clear();
-
-   int ne = mesh->GetNE();
-   for (int i = 0; i < ne; i++)
-   {
-      if (!el_attr_to_show[mesh->GetAttribute(i)-1]) { continue; }
-
-      center_ir.IntPoint(0) =
-         Geometries.GetCenter(mesh->GetElementBaseGeometry(i));
-      GetRefinedValues (i, center_ir, values, pointmat);
-
-      double xc = pointmat(0,0);
-      double yc = pointmat(1,0);
-      double uc = values(0);
-
-      double ds = GetElementLengthScale(i);
-      double dx = 0.05*ds;
-
-      double xx[3] = {xc,yc,uc};
-      DrawNumberedMarker(e_nums_buf,xx,dx,i);
-   }
-
    updated_bufs.emplace_back(&e_nums_buf);
 }
 
 void VisualizationSceneSolution::PrepareVertexNumbering()
 {
+   auto offset = [&](const int e, const int v)
+   {
+      if (!offsets) { return v; }
+      if (legacy_parallel_numbering) { return v; }
+      const int rank = mesh->GetAttribute(e) - 1;
+      MFEM_VERIFY(rank >= 0 && rank < (int)offsets->size(),
+                  "Invalid rank for element " + std::to_string(e));
+      const int nverts = (*offsets)[rank].nverts;
+      MFEM_VERIFY(v >= nverts,
+                  "Invalid vertex " + std::to_string(v) +
+                  " for element " + std::to_string(e));
+      return v - nverts;
+   };
+
+   v_nums_buf.clear();
+   DenseMatrix pointmat;
+   Array<int> vertices;
+
    if (shading == Shading::Noncomforming)
    {
-      PrepareVertexNumbering2();
+      Vector values;
+      for (int i = 0; i < mesh->GetNE(); i++)
+      {
+         if (!el_attr_to_show[mesh->GetAttribute(i)-1]) { continue; }
+         mesh->GetElementVertices (i, vertices);
+         const IntegrationRule &vert_ir =
+            *Geometries.GetVertices(mesh->GetElementBaseGeometry(i));
+         GetRefinedValues (i, vert_ir, values, pointmat);
+         const double xs = 0.05 * GetElementLengthScale(i);
+         for (int j = 0; j < values.Size(); j++)
+         {
+            double xv = pointmat(0, j);
+            double yv = pointmat(1, j);
+            double u = values[j];
+            double xx[3] = {xv, yv, u};
+            DrawNumberedMarker(v_nums_buf, xx, xs, offset(i, vertices[j]));
+         }
+      }
    }
    else
    {
-      PrepareVertexNumbering1();
-   }
-}
-
-void VisualizationSceneSolution::PrepareVertexNumbering1()
-{
-   v_nums_buf.clear();
-
-   DenseMatrix pointmat;
-   Array<int> vertices;
-
-   // Draw the vertices for each element. This is redundant, except when the
-   // elements or domains are shrunk.
-
-   const int ne = mesh->GetNE();
-   for (int k = 0; k < ne; k++)
-   {
-      mesh->GetPointMatrix (k, pointmat);
-      mesh->GetElementVertices (k, vertices);
-      int nv = vertices.Size();
-
-      ShrinkPoints(pointmat, k, 0, 0);
-
-      double ds = GetElementLengthScale(k);
-      double xs = 0.05*ds;
-
-      for (int j = 0; j < nv; j++)
+      // Draw the vertices for each element. This is redundant, except when the
+      // elements or domains are shrunk.
+      for (int e = 0; e < mesh->GetNE(); e++)
       {
-         double x = pointmat(0,j);
-         double y = pointmat(1,j);
-         double u = LogVal((*sol)(vertices[j]));
-
-         double xx[3] = {x,y,u};
-         DrawNumberedMarker(v_nums_buf,xx,xs,vertices[j]);
+         if (!el_attr_to_show[mesh->GetAttribute(e) - 1]) { continue; }
+         mesh->GetPointMatrix(e, pointmat);
+         mesh->GetElementVertices(e, vertices);
+         const int nv = vertices.Size();
+         const double xs = 0.05 * GetElementLengthScale(e);
+         for (int j = 0; j < nv; j++)
+         {
+            double x = pointmat(0,j);
+            double y = pointmat(1,j);
+            double u = LogVal((*sol)(vertices[j]));
+            double xx[3] = {x, y, u};
+            DrawNumberedMarker(v_nums_buf, xx, xs, offset(e, vertices[j]));
+         }
       }
    }
-
-   updated_bufs.emplace_back(&v_nums_buf);
-}
-
-void VisualizationSceneSolution::PrepareVertexNumbering2()
-{
-   DenseMatrix pointmat;
-   Vector values;
-   Array<int> vertices;
-
-   v_nums_buf.clear();
-
-   const int ne = mesh->GetNE();
-   for (int i = 0; i < ne; i++)
-   {
-      if (!el_attr_to_show[mesh->GetAttribute(i)-1]) { continue; }
-
-      mesh->GetElementVertices (i, vertices);
-
-      const IntegrationRule &vert_ir =
-         *Geometries.GetVertices(mesh->GetElementBaseGeometry(i));
-
-      GetRefinedValues (i, vert_ir, values, pointmat);
-
-      double ds = GetElementLengthScale(i);
-      double xs = 0.05*ds;
-
-      for (int j = 0; j < values.Size(); j++)
-      {
-         double xv = pointmat(0, j);
-         double yv = pointmat(1, j);
-
-         double u = values[j];
-
-         double xx[3] = {xv,yv,u};
-         DrawNumberedMarker(v_nums_buf,xx,xs,vertices[j]);
-      }
-   }
-
    updated_bufs.emplace_back(&v_nums_buf);
 }
 
@@ -1998,41 +2038,163 @@ void VisualizationSceneSolution::PrepareEdgeNumbering()
 
    f_nums_buf.clear();
 
+   Vector vals;
    DenseMatrix p;
-   Array<int> vertices;
-   Array<int> edges;
-   Array<int> edges_ori;
+   Array<int> vertices, edges, edges_ori;
 
-   const int ne = mesh->GetNE();
-   for (int k = 0; k < ne; k++)
+   auto offset = [&](const int e, const int i)
    {
-      mesh->GetElementEdges(k, edges, edges_ori);
+      const int edge = edges[i];
+      if (!offsets) { return edge; }
+      if (legacy_parallel_numbering) { return edge; }
+      const int rank = mesh->GetAttribute(e) - 1;
+      MFEM_VERIFY(rank >= 0 && rank < (int)offsets->size(),
+                  "Invalid rank for element " + std::to_string(e));
+      const int nedges = (*offsets)[rank].nedges;
+      MFEM_VERIFY(edge >= (int)nedges,
+                  "Invalid edge " + std::to_string(edge) +
+                  " for element " + std::to_string(e));
+      return edge - nedges;
+   };
 
-      double ds = GetElementLengthScale(k);
-      double xs = 0.05 * ds;
-
-      for (int i = 0; i < edges.Size(); i++)
+   if (shading == Shading::Flat || shading == Shading::Smooth)
+   {
+      for (int e = 0; e < mesh->GetNE(); e++)
       {
-         mesh->GetEdgeVertices(edges[i], vertices);
-
-         p.SetSize(mesh->Dimension(), vertices.Size());
-         p.SetCol(0, mesh->GetVertex(vertices[0]));
-         p.SetCol(1, mesh->GetVertex(vertices[1]));
-
-         ShrinkPoints(p, k, 0, 0);
-
-         const double m[2] = {0.5 * (p(0,0) + p(0,1)), 0.5 * (p(1,0) + p(1,1))};
-         // TODO: figure out something better...
-         double u = LogVal(0.5 * ((*sol)(vertices[0]) + (*sol)(vertices[1])));
-
-         double xx[3] = {m[0], m[1], u};
-         DrawNumberedMarker(f_nums_buf, xx, xs, edges[i]);
+         if (!el_attr_to_show[mesh->GetAttribute(e) - 1]) { continue; }
+         mesh->GetElementEdges(e, edges, edges_ori);
+         const double dx = 0.05 * GetElementLengthScale(e);
+         for (int i = 0; i < edges.Size(); i++)
+         {
+            mesh->GetEdgeVertices(edges[i], vertices);
+            p.SetSize(mesh->Dimension(), vertices.Size());
+            p.SetCol(0, mesh->GetVertex(vertices[0]));
+            p.SetCol(1, mesh->GetVertex(vertices[1]));
+            ShrinkPoints(p, e, 0, 0);
+            const double m[2] = {0.5 * (p(0,0) + p(0,1)), 0.5 * (p(1,0) + p(1,1))};
+            const double u = LogVal(0.5 * ((*sol)(vertices[0]) + (*sol)(vertices[1])));
+            const double xx[3] = {m[0], m[1], u};
+            DrawNumberedMarker(f_nums_buf, xx, dx, offset(e,i));
+         }
       }
    }
-
+   else if (shading == Shading::Noncomforming)
+   {
+      for (int e = 0; e < mesh->GetNE(); e++)
+      {
+         if (!el_attr_to_show[mesh->GetAttribute(e) - 1]) { continue; }
+         mesh->GetElementEdges(e, edges, edges_ori);
+         const auto dx = 0.05 * GetElementLengthScale(e);
+         const auto geom = mesh->GetElementBaseGeometry(e);
+         if (geom != Geometry::TRIANGLE && geom != Geometry::SQUARE)
+         {
+            std::cerr  << "Only TRIANGLE and SQUARE geometries are supported." << std::endl;
+            return;
+         }
+         const auto *RefG = GLVisGeometryRefiner.Refine(geom, 2, 2);
+         GetRefinedValues(e, RefG->RefPts, vals, p);
+         const int ij3[3] = { 1, 4, 3 }, ie3[3] = { 0, 1, 2 };
+         const int ij4[4] = { 1, 3, 5, 7 }, ie4[4] = { 0, 3, 1, 2 };
+         const int *ij = geom == Geometry::TRIANGLE ? ij3 : ij4;
+         const int *ie = geom == Geometry::TRIANGLE ? ie3 : ie4;
+         for (int i = 0; i < edges.Size(); i++)
+         {
+            const int j = ij[i];
+            const double xx[3] = { p(0,j), p(1,j), vals(j) };
+            DrawNumberedMarker(f_nums_buf, xx, dx, offset(e,ie[i]));
+         }
+      }
+   }
+   else { MFEM_ABORT("Shading not supported"); }
    updated_bufs.emplace_back(&f_nums_buf);
 }
 
+void VisualizationSceneSolution::PrepareDofNumbering()
+{
+   Vector vals;
+   DenseMatrix tr;
+   Array<int> dofs;
+
+   d_nums_buf.clear();
+
+   auto offset = [&](const int e, const int q)
+   {
+      if (!offsets) { return dofs[q]; }
+      if (legacy_parallel_numbering) { return dofs[q]; }
+      const int rank = mesh->GetAttribute(e) - 1;
+      return (*offsets)[rank][ {e,q}];
+   };
+
+   if (shading == Shading::Flat || shading == Shading::Smooth)
+   {
+      H1_FECollection h1_fec(1, mesh->Dimension());
+      FiniteElementSpace h1_fes(mesh, &h1_fec);
+      MFEM_VERIFY(sol && sol->Size() == h1_fes.GetNDofs(),
+                  "Flat space does not match the solution size");
+      GridFunction h1_sol(&h1_fes, sol->GetData());
+      const auto *fes = rsol ? rsol->FESpace() : &h1_fes;
+      const int sdim = mesh->SpaceDimension();
+      IsoparametricTransformation Tr;
+
+      for (int e = 0; e < mesh->GetNE(); e++)
+      {
+         if (!el_attr_to_show[mesh->GetAttribute(e) - 1]) { continue; }
+         const auto dx = 0.05 * GetElementLengthScale(e);
+         const auto &ir = fes->GetFE(e)->GetNodes();
+         const Element *el = mesh->GetElement(e);
+         const int nv = el->GetNVertices();
+         const int *verts = el->GetVertices();
+         DenseMatrix &pm = Tr.GetPointMat();
+         pm.SetSize(sdim, nv);
+         for (int v = 0; v < nv; v++)
+         {
+            const real_t *vert = mesh->GetVertex(verts[v]);
+            for (int d = 0; d < sdim; d++)
+            {
+               pm(d, v) = vert[d];
+            }
+         }
+         Tr.SetFE(mesh->GetTransformationFEforElementType(el->GetType()));
+         Tr.Transform(ir, tr);
+         fes->GetElementDofs(e, dofs);
+         fes->AdjustVDofs(dofs);
+
+         for (int q = 0; q < ir.GetNPoints(); q++)
+         {
+            const real_t z = h1_sol.GetValue(e, ir.IntPoint(q));
+            const real_t x[3] = {tr(0,q), tr(1,q), z};
+            DrawNumberedMarker(d_nums_buf, x, dx, offset(e,q));
+         }
+      }
+   }
+   else if (shading == Shading::Noncomforming)
+   {
+      MFEM_VERIFY(rsol, "Solution required for Noncomforming dof numbering");
+      const auto *rsol_fes = rsol->FESpace();
+      MFEM_VERIFY(rsol->Size() == rsol_fes->GetNDofs(),
+                  "FE space does not match the rsol size");
+
+      for (int e = 0; e < mesh->GetNE(); e++)
+      {
+         if (!el_attr_to_show[mesh->GetAttribute(e) - 1]) { continue; }
+         const auto dx = 0.05 * GetElementLengthScale(e);
+         const auto &ir = rsol_fes->GetFE(e)->GetNodes();
+         GetRefinedValues(e, ir, vals, tr, true);
+         rsol->FESpace()->GetElementDofs(e, dofs);
+         rsol->FESpace()->AdjustVDofs(dofs);
+
+         for (int q = 0; q < ir.GetNPoints(); q++)
+         {
+            const real_t z = vals[q];
+            const real_t x[3] = {tr(0,q), tr(1,q), z};
+            DrawNumberedMarker(d_nums_buf, x, dx, offset(e,q));
+         }
+      }
+   }
+   else { MFEM_ABORT("Shading not supported"); }
+   updated_bufs.emplace_back(&d_nums_buf);
+}
+
 void VisualizationSceneSolution::PrepareOrderingCurve()
 {
    bool color = draworder < 3;
@@ -2129,22 +2291,35 @@ void VisualizationSceneSolution::PrepareNumbering(bool invalidate)
       e_nums_buf_ready = false;
       v_nums_buf_ready = false;
       f_nums_buf_ready = false;
+      d_nums_buf_ready = false;
    }
-   if (drawnums == 1 && !e_nums_buf_ready)
+   else
+   {
+      static const char *numbering[(int)Numbering::MAX] =
+      { "None", "Elements", "Edges", "Vertices", "DOFs" };
+      std::cout << "Numbering : " << numbering[(int)drawnums] << std::endl;
+   }
+
+   if (drawnums == Numbering::ELEMENTS && !e_nums_buf_ready)
    {
       PrepareElementNumbering();
       e_nums_buf_ready = true;
    }
-   if (drawnums == 2 && !f_nums_buf_ready)
+   if (drawnums == Numbering::EDGES && !f_nums_buf_ready)
    {
       PrepareEdgeNumbering();
       f_nums_buf_ready = true;
    }
-   if (drawnums == 3 && !v_nums_buf_ready)
+   if (drawnums == Numbering::VERTICES && !v_nums_buf_ready)
    {
       PrepareVertexNumbering();
       v_nums_buf_ready = true;
    }
+   if (drawnums == Numbering::DOFS && !d_nums_buf_ready)
+   {
+      PrepareDofNumbering();
+      d_nums_buf_ready = true;
+   }
 }
 
 void VisualizationSceneSolution::PrepareLines2()
@@ -2554,18 +2729,27 @@ gl3::SceneInfo VisualizationSceneSolution::GetSceneObjs()
    }
 
    // draw numberings
-   if (drawnums == 1)
+   if (drawnums == Numbering::ELEMENTS)
    {
       scene.queue.emplace_back(params, &e_nums_buf);
    }
-   else if (drawnums == 2)
+   else if (drawnums == Numbering::EDGES)
    {
       scene.queue.emplace_back(params, &f_nums_buf);
    }
-   else if (drawnums == 3)
+   else if (drawnums == Numbering::VERTICES)
    {
       scene.queue.emplace_back(params, &v_nums_buf);
    }
+   else if (drawnums == Numbering::DOFS)
+   {
+      scene.queue.emplace_back(params, &d_nums_buf);
+   }
+   else
+   {
+      MFEM_VERIFY(drawnums == Numbering::NONE,
+                  "Unsupported drawnums value: " << (int)drawnums);
+   }
 
    // draw orderings -- "black" modes
    if (draworder == 3)
