@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,13 +9,14 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#include "vsvector.hpp"
+
 #include <cstdlib>
 #include <iostream>
 #include <limits>
 #include <cmath>
 
-#include "mfem.hpp"
-#include "visual.hpp"
+#include "threads.hpp"
 
 using namespace mfem;
 using namespace std;
@@ -50,9 +51,16 @@ std::string VisualizationSceneVector::GetHelpString() const
       << "| o -  (De)refine elem. (NC shading) |" << endl
       << "| O -  Switch 'o' func. (NC shading) |" << endl
       << "| p/P  Cycle through color palettes  |" << endl
-      << "| q -  Quits                         |" << endl
-      << "| Q -  Cycle quadrature data mode    |" << endl
-      << "| r -  Reset the plot to 3D view     |" << endl
+      << "| q -  Quits                         |" << endl;
+   if (win.data_state.quad_f)
+   {
+      os << "| Q -  Cycle quadrature data mode    |" << endl;
+   }
+   else if (win.data_state.cgrid_f)
+   {
+      os << "| Q -  Cycle complex data mode       |" << endl;
+   }
+   os << "| r -  Reset the plot to 3D view     |" << endl
       << "| R -  Reset the plot to 2D view     |" << endl
       << "| s -  Turn on/off unit cube scaling |" << endl
       << "| S -  Take snapshot/Record a movie  |" << endl
@@ -84,8 +92,13 @@ std::string VisualizationSceneVector::GetHelpString() const
       << "| *,/  Scale up/down                 |" << endl
       << "| +/-  Change z-scaling              |" << endl
       << "| . -  Start/stop spinning           |" << endl
-      << "| 0/Enter - Spinning speed and dir.  |" << endl
-      << "+------------------------------------+" << endl
+      << "| 0/Enter - Spinning speed and dir.  |" << endl;
+   if (win.data_state.cgrid_f)
+   {
+      os << "| Alt+. -  Start/stop phase anim.    |" << endl
+         << "| Alt+0/Enter - Phase anim. speed    |" << endl;
+   }
+   os << "+------------------------------------+" << endl
       << "| Mouse                              |" << endl
       << "+------------------------------------+" << endl
       << "| left   btn    - Rotation           |" << endl
@@ -102,9 +115,7 @@ std::string VisualizationSceneVector::GetHelpString() const
    return os.str();
 }
 
-thread_local VisualizationSceneVector  * vsvector;
-extern thread_local VisualizationScene * locscene;
-extern thread_local VisualizationSceneSolution * vssol;
+static thread_local VisualizationSceneVector  * vsvector;
 extern thread_local GeometryRefiner GLVisGeometryRefiner;
 
 thread_local int ianim = 0;
@@ -256,56 +267,46 @@ const char *Vec2ScalarNames[7] =
    "curl", "anisotropy"
 };
 
-VisualizationSceneVector::VisualizationSceneVector(Mesh & m,
-                                                   Vector & sx, Vector & sy, Mesh *mc)
+VisualizationSceneVector::VisualizationSceneVector(Window &win_)
+   : VisualizationSceneSolution(win_, false)
 {
-   mesh = &m;
-   mesh_coarse = mc;
-   solx = &sx;
-   soly = &sy;
-
-   sol  = new Vector(mesh -> GetNV());
-
-   VecGridF = NULL;
-
-   Init();
-}
-
-VisualizationSceneVector::VisualizationSceneVector(GridFunction &vgf)
-{
-   FiniteElementSpace *fes = vgf.FESpace();
-   if (fes == NULL || vgf.VectorDim() != 2)
+   if (win.data_state.grid_f)
    {
-      cout << "VisualizationSceneVector::VisualizationSceneVector" << endl;
-      exit(1);
-   }
+      FiniteElementSpace *fes = win.data_state.grid_f->FESpace();
+      if (fes == NULL || win.data_state.grid_f->VectorDim() != 2)
+      {
+         cout << "VisualizationSceneVector::VisualizationSceneVector" << endl;
+         exit(1);
+      }
 
-   VecGridF = &vgf;
+      VecGridF = win.data_state.grid_f.get();
 
-   mesh = fes->GetMesh();
+      solx = new Vector(mesh -> GetNV());
+      soly = new Vector(mesh -> GetNV());
 
-   solx = new Vector(mesh -> GetNV());
-   soly = new Vector(mesh -> GetNV());
-
-   vgf.GetNodalValues (*solx, 1);
-   vgf.GetNodalValues (*soly, 2);
-
-   sol = new Vector(mesh -> GetNV());
-
-   //  VisualizationSceneSolution::Init()  sets rsol = NULL !
-   {
-      Init();
-      SetGridFunction(vgf);
-   }
-
-   mesh->GetNodes(vc0);
-   if (vc0.Size() != vgf.Size())
-   {
-      vc0.Destroy();
+      VecGridF->GetNodalValues(*solx, 1);
+      VecGridF->GetNodalValues(*soly, 2);
    }
    else
    {
-      vc0 += vgf;
+      sol  = new Vector(mesh -> GetNV());
+      solx = win.data_state.solx.get();
+      soly = win.data_state.soly.get();
+   }
+
+   Init();
+
+   if (VecGridF)
+   {
+      mesh->GetNodes(vc0);
+      if (vc0.Size() != VecGridF->Size())
+      {
+         vc0.Destroy();
+      }
+      else
+      {
+         vc0 += *VecGridF;
+      }
    }
 }
 
@@ -357,7 +358,7 @@ void VisualizationSceneVector::CycleVec2Scalar(int print)
    for (i = 0; Vec2Scalar != Vec2ScalarFunctions[i]; i++)
       ;
 
-   if (VecGridF->FESpace()->GetVDim() == 1)
+   if (VecGridF && VecGridF->FESpace()->GetVDim() == 1)
    {
       if (dynamic_cast<const ND_FECollection*>(VecGridF->FESpace()->FEColl()))
       {
@@ -370,10 +371,14 @@ void VisualizationSceneVector::CycleVec2Scalar(int print)
          i = (i + 1) % 5;
       }
    }
-   else
+   else if (VecGridF)
    {
       i = (i + 1) % 7;
    }
+   else
+   {
+      i = (i + 1) % 4;
+   }
 
    if (print)
    {
@@ -381,7 +386,7 @@ void VisualizationSceneVector::CycleVec2Scalar(int print)
    }
 
    Vec2Scalar = Vec2ScalarFunctions[i];
-   extra_caption = Vec2ScalarNames[i];
+   win.extra_caption = Vec2ScalarNames[i];
 
    for (i = 0; i < mesh->GetNV(); i++)
    {
@@ -408,7 +413,21 @@ void VisualizationSceneVector::CycleVec2Scalar(int print)
    }
 }
 
-void VisualizationSceneVector::NewMeshAndSolution(GridFunction &vgf, Mesh *mc)
+void VisualizationSceneVector::NewMeshAndSolution(const DataState &s)
+{
+   if (VecGridF && s.grid_f)
+   {
+      NewMeshAndSolution(s.mesh.get(), s.mesh_quad.get(), solx, soly, s.grid_f.get());
+   }
+   else
+   {
+      NewMeshAndSolution(s.mesh.get(), s.mesh_quad.get(), s.solx.get(), s.soly.get());
+   }
+}
+
+void VisualizationSceneVector::NewMeshAndSolution(
+   Mesh *new_m, Mesh *new_mc, Vector *new_sol_x, Vector *new_sol_y,
+   GridFunction *vgf)
 {
    delete sol;
 
@@ -419,10 +438,9 @@ void VisualizationSceneVector::NewMeshAndSolution(GridFunction &vgf, Mesh *mc)
    }
 
    Mesh *old_m = mesh;
-   Mesh *new_mesh = vgf.FESpace()->GetMesh();
-   mesh = new_mesh;
-   mesh_coarse = mc;
-   VecGridF = &vgf;
+   mesh = new_m;
+   mesh_coarse = new_mc;
+   VecGridF = vgf;
 
    // If the number of elements changes, recompute the refinement factor
    if (mesh->GetNE() != old_m->GetNE())
@@ -441,20 +459,28 @@ void VisualizationSceneVector::NewMeshAndSolution(GridFunction &vgf, Mesh *mc)
       }
    }
 
-   solx = new Vector(mesh->GetNV());
-   soly = new Vector(mesh->GetNV());
-
-   vgf.GetNodalValues(*solx, 1);
-   vgf.GetNodalValues(*soly, 2);
-
-   mesh->GetNodes(vc0);
-   if (vc0.Size() != vgf.Size())
+   if (vgf)
    {
-      vc0.Destroy();
+      solx = new Vector(mesh->GetNV());
+      soly = new Vector(mesh->GetNV());
+
+      vgf->GetNodalValues(*solx, 1);
+      vgf->GetNodalValues(*soly, 2);
+
+      mesh->GetNodes(vc0);
+      if (vc0.Size() != vgf->Size())
+      {
+         vc0.Destroy();
+      }
+      else
+      {
+         vc0 += *vgf;
+      }
    }
    else
    {
-      vc0 += vgf;
+      solx = new_sol_x;
+      soly = new_sol_y;
    }
 
    sol = new Vector(mesh->GetNV());
@@ -463,9 +489,9 @@ void VisualizationSceneVector::NewMeshAndSolution(GridFunction &vgf, Mesh *mc)
       (*sol)(i) = Vec2Scalar((*solx)(i), (*soly)(i));
    }
 
-   VisualizationSceneSolution::NewMeshAndSolution(mesh, mesh_coarse, sol, &vgf);
+   VisualizationSceneSolution::NewMeshAndSolution(mesh, mesh_coarse, sol, vgf);
 
-   if (autoscale)
+   if (autoscale != VisualizationSceneScalarData::Autoscale::None)
    {
       if (Vec2Scalar == VecLength)
       {
@@ -488,7 +514,7 @@ void VisualizationSceneVector::Init()
    ArrowScale = 1.0;
    RefineFactor = 1;
    Vec2Scalar = VecLength;
-   extra_caption = Vec2ScalarNames[0];
+   win.extra_caption = Vec2ScalarNames[0];
 
    for (int i = 0; i < mesh->GetNV(); i++)
    {
@@ -523,17 +549,21 @@ void VisualizationSceneVector::Init()
 
 VisualizationSceneVector::~VisualizationSceneVector()
 {
-   delete sol;
-
    if (VecGridF)
    {
       delete soly;
       delete solx;
    }
+   else
+   {
+      delete sol;
+   }
 }
 
-void VisualizationSceneVector::GetRefinedValues(
-   int i, const IntegrationRule &ir, Vector &vals, DenseMatrix &tr)
+void VisualizationSceneVector::GetRefinedValues(const int i,
+                                                const IntegrationRule &ir,
+                                                Vector &vals, DenseMatrix &tr,
+                                                const bool do_shrink)
 {
    if (drawelems < 2)
    {
@@ -638,7 +668,7 @@ void VisualizationSceneVector::GetRefinedValues(
          vals(j) = _LogVal(vals(j));
       }
 
-   if (shrink != 1.0 || shrinkmat != 1.0)
+   if (do_shrink && (shrink != 1.0 || shrinkmat != 1.0))
    {
       ShrinkPoints(tr, i, 0, 0);
    }
@@ -892,6 +922,48 @@ int VisualizationSceneVector::GetFunctionAutoRefineFactor()
    return VisualizationSceneScalarData::GetFunctionAutoRefineFactor(*VecGridF);
 }
 
+void VisualizationSceneVector::FindNewBox(bool prepare)
+{
+   VisualizationSceneSolution::FindNewBox(bb.x, bb.y, bb.z);
+
+   win.data_state.FindValueRange(minv, maxv,
+   [this](const Vector &x) { return Vec2Scalar(x(0), x(1)); },
+   [this](double v) { return LogVal(v); });
+
+   if (minv == 0. && maxv == 0.)
+   {
+      minv = bb.z[0];
+      maxv = bb.z[1];
+   }
+
+   FixValueRange();
+
+   bb.z[0] = minv;
+   bb.z[1] = maxv;
+
+   SetNewScalingFromBox(); // UpdateBoundingBox minus PrepareAxes
+   UpdateValueRange(prepare);
+}
+
+void VisualizationSceneVector::FindNewValueRange(bool prepare)
+{
+   win.data_state.FindValueRange(minv, maxv,
+   [this](const Vector &x) { return Vec2Scalar(x(0), x(1)); },
+   [this](double v) { return LogVal(v); });
+
+   if (minv == 0. && maxv == 0.)
+   {
+      double rx[2], ry[2], rv[2];
+      VisualizationSceneSolution::FindNewBox(rx, ry, rv);
+      minv = rv[0];
+      maxv = rv[1];
+   }
+
+   FixValueRange();
+
+   UpdateValueRange(prepare);
+}
+
 void VisualizationSceneVector::PrepareVectorField()
 {
    int rerun;
@@ -1020,14 +1092,23 @@ gl3::SceneInfo VisualizationSceneVector::GetSceneObjs()
    }
 
    // draw numberings
-   if (drawnums == 1)
+   if (drawnums == Numbering::ELEMENTS)
    {
       scene.queue.emplace_back(params, &e_nums_buf);
    }
-   else if (drawnums == 2)
+   else if (drawnums == Numbering::VERTICES)
    {
       scene.queue.emplace_back(params, &v_nums_buf);
    }
+   else if (drawnums == Numbering::EDGES)
+   {
+      scene.queue.emplace_back(params, &f_nums_buf);
+   }
+   else if (drawnums == Numbering::DOFS)
+   {
+      scene.queue.emplace_back(params, &d_nums_buf);
+   }
+   else { /* Numbering::NONE */ }
 
    if (drawvector == 1 || drawvector > 3)
    {
