@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2026, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -9,13 +9,14 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#include "vsvector3d.hpp"
+
 #include <cstdlib>
 #include <iostream>
 #include <cmath>
 #include <limits>
 
-#include "mfem.hpp"
-#include "visual.hpp"
+#include "threads.hpp"
 
 using namespace mfem;
 using namespace std;
@@ -60,9 +61,16 @@ std::string VisualizationSceneVector3d::GetHelpString() const
       << "| n -  Displacements step forward    |" << endl
       << "| o/O  (De)refine elem, disc shading |" << endl
       << "| p/P  Cycle through color palettes  |" << endl
-      << "| q -  Quits                         |" << endl
-      << "| Q -  Cycle quadrature data mode    |" << endl
-      << "| r -  Reset the plot to 3D view     |" << endl
+      << "| q -  Quits                         |" << endl;
+   if (win.data_state.quad_f)
+   {
+      os << "| Q -  Cycle quadrature data mode    |" << endl;
+   }
+   else if (win.data_state.cgrid_f)
+   {
+      os << "| Q -  Cycle complex data mode       |" << endl;
+   }
+   os << "| r -  Reset the plot to 3D view     |" << endl
       << "| R -  Reset the plot to 2D view     |" << endl
       << "| s -  Turn on/off unit cube scaling |" << endl
       << "| S -  Take snapshot/Record a movie  |" << endl
@@ -96,8 +104,13 @@ std::string VisualizationSceneVector3d::GetHelpString() const
       << "| *,/  Scale up/down                 |" << endl
       << "| +/-  Change z-scaling              |" << endl
       << "| . -  Start/stop spinning           |" << endl
-      << "| 0/Enter - Spinning speed and dir.  |" << endl
-      << "+------------------------------------+" << endl
+      << "| 0/Enter - Spinning speed and dir.  |" << endl;
+   if (win.data_state.cgrid_f)
+   {
+      os << "| Alt+. -  Start/stop phase anim.    |" << endl
+         << "| Alt+0/Enter - Phase anim. speed    |" << endl;
+   }
+   os << "+------------------------------------+" << endl
       << "| Mouse                              |" << endl
       << "+------------------------------------+" << endl
       << "| left   btn    - Rotation           |" << endl
@@ -114,8 +127,8 @@ std::string VisualizationSceneVector3d::GetHelpString() const
    return os.str();
 }
 
-thread_local VisualizationSceneVector3d  *vsvector3d;
-extern thread_local VisualizationScene *locscene;
+static thread_local VisualizationSceneVector3d  *vsvector3d;
+static thread_local Window *window;
 extern thread_local GeometryRefiner GLVisGeometryRefiner;
 
 static void KeyDPressed()
@@ -150,13 +163,13 @@ static void KeyBPressed()
 
 static void KeyrPressed()
 {
-   locscene -> spinning = 0;
+   window->vs -> spinning = 0;
    RemoveIdleFunc(MainLoop);
    vsvector3d -> CenterObject();
-   locscene -> ViewAngle = 45.0;
-   locscene -> ViewScale = 1.0;
-   locscene -> ViewCenterX = 0.0;
-   locscene -> ViewCenterY = 0.0;
+   window->vs -> ViewAngle = 45.0;
+   window->vs -> ViewScale = 1.0;
+   window->vs -> ViewCenterX = 0.0;
+   window->vs -> ViewCenterY = 0.0;
    vsvector3d -> ianim = vsvector3d -> ianimd = 0;
    vsvector3d -> Prepare();
    vsvector3d -> PrepareLines();
@@ -167,7 +180,7 @@ static void KeyrPressed()
 
 static void KeyRPressed()
 {
-   locscene->spinning = 0;
+   window->vs->spinning = 0;
    RemoveIdleFunc(MainLoop);
    vsvector3d -> ianim = vsvector3d -> ianimd = 0;
    vsvector3d -> Prepare();
@@ -275,8 +288,8 @@ void VisualizationSceneVector3d::ToggleVectorField(int i)
    PrepareVectorField();
 }
 
-static const char *scal_func_name[] =
-{"magnitude", "x-component", "y-component", "z-component"};
+const char *VisualizationSceneVector3d::scal_func_name[(int)ScalarFunction::Max]
+   = {"magnitude", "x-component", "y-component", "z-component"};
 
 void VisualizationSceneVector3d::SetScalarFunction()
 {
@@ -284,7 +297,7 @@ void VisualizationSceneVector3d::SetScalarFunction()
 
    switch (scal_func)
    {
-      case 0: // magnitude
+      case ScalarFunction::Magnitude:
          for (int i = 0; i < sol->Size(); i++)
             (*sol)(i) = sqrt((*solx)(i) * (*solx)(i) +
                              (*soly)(i) * (*soly)(i) +
@@ -305,7 +318,7 @@ void VisualizationSceneVector3d::SetScalarFunction()
             }
          }
          break;
-      case 1: // x-component
+      case ScalarFunction::Component_X:
          *sol = *solx;
          if (GridF)
             for (int i = 0; i < GridF->Size(); i++)
@@ -313,7 +326,7 @@ void VisualizationSceneVector3d::SetScalarFunction()
                (*GridF)(i) = (*VecGridF)(fes->DofToVDof(i, 0));
             }
          break;
-      case 2: // y-component
+      case ScalarFunction::Component_Y:
          *sol = *soly;
          if (GridF)
             for (int i = 0; i < GridF->Size(); i++)
@@ -321,7 +334,7 @@ void VisualizationSceneVector3d::SetScalarFunction()
                (*GridF)(i) = (*VecGridF)(fes->DofToVDof(i, 1));
             }
          break;
-      case 3: // z-component
+      case ScalarFunction::Component_Z:
          *sol = *solz;
          if (GridF)
             for (int i = 0; i < GridF->Size(); i++)
@@ -329,73 +342,66 @@ void VisualizationSceneVector3d::SetScalarFunction()
                (*GridF)(i) = (*VecGridF)(fes->DofToVDof(i, 2));
             }
          break;
+      default:
+         cerr << "Unknown scalar representation" << endl;
+         return;
    }
-   extra_caption = scal_func_name[scal_func];
+   win.extra_caption = scal_func_name[(int)scal_func];
 }
 
 void VisualizationSceneVector3d::ToggleScalarFunction()
 {
-   scal_func = (scal_func + 1) % 4;
-   cout << "Displaying " << scal_func_name[scal_func] << endl;
+   scal_func = (ScalarFunction)(((int)scal_func + 1) % ((int)ScalarFunction::Max));
+   cout << "Displaying " << scal_func_name[(int)scal_func] << endl;
    SetScalarFunction();
    FindNewValueRange(true);
 }
 
-VisualizationSceneVector3d::VisualizationSceneVector3d(Mesh &m, Vector &sx,
-                                                       Vector &sy, Vector &sz, Mesh *mc)
+VisualizationSceneVector3d::VisualizationSceneVector3d(Window &win_)
+   : VisualizationSceneSolution3d(win_, false)
 {
-   mesh = &m;
-   mesh_coarse = mc;
-   solx = &sx;
-   soly = &sy;
-   solz = &sz;
-
-   sol = new Vector(mesh->GetNV());
-
-   sfes = NULL;
-   VecGridF = NULL;
-
-   Init();
-}
-
-VisualizationSceneVector3d::VisualizationSceneVector3d(GridFunction &vgf,
-                                                       Mesh *mc)
-{
-   FiniteElementSpace *fes = vgf.FESpace();
-   if (fes == NULL || fes->GetVDim() != 3)
+   if (win.data_state.grid_f)
    {
-      cout << "VisualizationSceneVector3d::VisualizationSceneVector3d" << endl;
-      exit(1);
+      FiniteElementSpace *fes = win.data_state.grid_f->FESpace();
+      if (fes == NULL || fes->GetVDim() != 3)
+      {
+         cout << "VisualizationSceneVector3d::VisualizationSceneVector3d" << endl;
+         exit(1);
+      }
+
+      VecGridF = win.data_state.grid_f.get();
+
+      sfes = new FiniteElementSpace(mesh, fes->FEColl(), 1, fes->GetOrdering());
+      GridF = new GridFunction(sfes);
+
+      solx = new Vector(mesh->GetNV());
+      soly = new Vector(mesh->GetNV());
+      solz = new Vector(mesh->GetNV());
+
+      VecGridF->GetNodalValues(*solx, 1);
+      VecGridF->GetNodalValues(*soly, 2);
+      VecGridF->GetNodalValues(*solz, 3);
+   }
+   else
+   {
+      sol = new Vector(mesh->GetNV());
+      solx = win.data_state.solx.get();
+      soly = win.data_state.soly.get();
+      solz = win.data_state.solz.get();
    }
-
-   VecGridF = &vgf;
-
-   mesh = fes->GetMesh();
-   mesh_coarse = mc;
-
-   sfes = new FiniteElementSpace(mesh, fes->FEColl(), 1, fes->GetOrdering());
-   GridF = new GridFunction(sfes);
-
-   solx = new Vector(mesh->GetNV());
-   soly = new Vector(mesh->GetNV());
-   solz = new Vector(mesh->GetNV());
-
-   vgf.GetNodalValues(*solx, 1);
-   vgf.GetNodalValues(*soly, 2);
-   vgf.GetNodalValues(*solz, 3);
-
-   sol = new Vector(mesh->GetNV());
 
    Init();
 }
 
 void VisualizationSceneVector3d::Init()
 {
+   window = &win;
+
    key_r_state = 0;
 
    drawdisp = 0;
    drawvector = 0;
-   scal_func = 0;
+   scal_func = ScalarFunction::Magnitude;
 
    ianim = ianimd = 0;
    ianimmax = 10;
@@ -460,8 +466,6 @@ int VisualizationSceneVector3d::GetFunctionAutoRefineFactor()
 
 VisualizationSceneVector3d::~VisualizationSceneVector3d()
 {
-   delete sol;
-
    if (VecGridF)
    {
       delete solz;
@@ -470,10 +474,29 @@ VisualizationSceneVector3d::~VisualizationSceneVector3d()
       delete GridF;
       delete sfes;
    }
+   else
+   {
+      delete sol;
+   }
+}
+
+void VisualizationSceneVector3d::NewMeshAndSolution(const DataState &s)
+{
+   if (VecGridF && s.grid_f)
+   {
+      NewMeshAndSolution(s.mesh.get(), s.mesh_quad.get(), solx, soly, solz,
+                         s.grid_f.get());
+   }
+   else
+   {
+      NewMeshAndSolution(s.mesh.get(), s.mesh_quad.get(), s.solx.get(), s.soly.get(),
+                         s.solz.get());
+   }
 }
 
 void VisualizationSceneVector3d::NewMeshAndSolution(
-   Mesh *new_m, Mesh *new_mc, GridFunction *new_v)
+   Mesh *new_m, Mesh *new_mc, Vector *new_sol_x, Vector *new_sol_y,
+   Vector *new_sol_z, GridFunction *new_v)
 {
    delete sol;
    if (VecGridF)
@@ -508,21 +531,30 @@ void VisualizationSceneVector3d::NewMeshAndSolution(
       }
    }
 
-   FiniteElementSpace *new_fes = new_v->FESpace();
 
    FindNodePos();
 
-   sfes = new FiniteElementSpace(mesh, new_fes->FEColl(), 1,
-                                 new_fes->GetOrdering());
-   GridF = new GridFunction(sfes);
+   if (new_v)
+   {
+      FiniteElementSpace *new_fes = new_v->FESpace();
+      sfes = new FiniteElementSpace(mesh, new_fes->FEColl(), 1,
+                                    new_fes->GetOrdering());
+      GridF = new GridFunction(sfes);
 
-   solx = new Vector(mesh->GetNV());
-   soly = new Vector(mesh->GetNV());
-   solz = new Vector(mesh->GetNV());
+      solx = new Vector(mesh->GetNV());
+      soly = new Vector(mesh->GetNV());
+      solz = new Vector(mesh->GetNV());
 
-   VecGridF->GetNodalValues(*solx, 1);
-   VecGridF->GetNodalValues(*soly, 2);
-   VecGridF->GetNodalValues(*solz, 3);
+      VecGridF->GetNodalValues(*solx, 1);
+      VecGridF->GetNodalValues(*soly, 2);
+      VecGridF->GetNodalValues(*solz, 3);
+   }
+   else
+   {
+      solx = new_sol_x;
+      soly = new_sol_y;
+      solz = new_sol_z;
+   }
 
    sol = new Vector(mesh->GetNV());
 
@@ -539,6 +571,52 @@ void VisualizationSceneVector3d::NewMeshAndSolution(
    PrepareDisplacedMesh();
 }
 
+void VisualizationSceneVector3d::FindNewValueRange(bool prepare)
+{
+   function<double(const Vector &)> vec2scal;
+   switch (scal_func)
+   {
+      case ScalarFunction::Magnitude:
+         vec2scal = [](const Vector &v) { return v.Norml2(); };
+         break;
+      case ScalarFunction::Component_X:
+         vec2scal = [](const Vector &v) { return v(0); };
+         break;
+      case ScalarFunction::Component_Y:
+         vec2scal = [](const Vector &v) { return v(1); };
+         break;
+      case ScalarFunction::Component_Z:
+         vec2scal = [](const Vector &v) { return v(2); };
+         break;
+      default:
+         cerr << "Unknown scalar representation" << endl;
+         return;
+   }
+
+   win.data_state.FindValueRange(minv, maxv, vec2scal);
+
+   if (minv == 0. && maxv == 0.)
+   {
+      int map_type = (GridF) ?
+                     GridF->FESpace()->FEColl()->GetMapType(mesh->Dimension()) :
+                     FiniteElement::VALUE;
+
+      if (shading < Shading::Noncomforming || map_type != (int)FiniteElement::VALUE)
+      {
+         minv = sol->Min();
+         maxv = sol->Max();
+      }
+      else
+      {
+         minv = GridF->Min();
+         maxv = GridF->Max();
+      }
+      return;
+   }
+   FixValueRange();
+   UpdateValueRange(prepare);
+}
+
 void VisualizationSceneVector3d::PrepareFlat()
 {
    int i, j;
